//------------------------------------------------------------------------------
// File:		input.c
// Description: generic input handling functions
//
// Copyright (C) 2006 JasonV
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//------------------------------------------------------------------------------



// included files
//--------------------------------------------------
#include "input.h"



// globals
//--------------------------------------------------
SceCtrlData CurController;
SceCtrlData PrevController;



// functions
//--------------------------------------------------
void InitInput( void )
{
	sceCtrlSetSamplingCycle(0);
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

	memset( &PrevController, 0, sizeof(PrevController) );
	memset( &CurController, 0, sizeof(CurController) );

	ReadInput();
}



int AnalogStateX( void )
{
	int deadzone = 20;
	int val = 0;
	if( CurController.Lx > (127+deadzone) )
	{
		val = (CurController.Lx - 127 - deadzone) * 100 / (127 - deadzone);
	}
	else if( CurController.Lx < (127-deadzone) )
	{
		val = (127 - CurController.Lx - deadzone) * -100 / (127 - deadzone);
	}
	return val;
}



int AnalogStateY( void )
{
	int deadzone = 20;
	int val = 0;
	if( CurController.Ly > (127+deadzone) )
	{
		val = (CurController.Ly - 127 - deadzone) * 100 / (127 - deadzone);
	}
	else if( CurController.Ly < (127-deadzone) )
	{
		val = (127 - CurController.Ly - deadzone) * -100 / (127 - deadzone);
	}
	return val;
}



bool ButtonPressed( int button )
{
	return ( (button&CurController.Buttons)>0 && (button&PrevController.Buttons)==0 );
}



bool ButtonReleased( int button )
{
	return ( (button&CurController.Buttons)==0 && (button&PrevController.Buttons)>0 );
}



bool ButtonHeld( int button )
{
	return ( (button&CurController.Buttons)>0 && (button&PrevController.Buttons)>0 );
}



void ReadInput( void )
{
	PrevController = CurController;
	sceCtrlReadBufferPositive( &CurController, 1 );
}


