//
// Copyright (C) 2006 JasonV
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include <pspgu.h>
#include <malloc.h>
#include <fastmath.h>
#include <stdlib.h>
#include <pspdisplay.h>
#include <psputils.h>
#include <stdio.h>
#include "opendir.h"
#include "gu_funcs.h"
#include <pspdebug.h>
#include "../types.h"
#include <png.h>



// Constants
//--------------------------------------------------
#define SCREEN_WIDTH	480
#define SCREEN_HEIGHT	272

#define FRAME_BUFFER_WIDTH 512
#define FRAME_BUFFER_SIZE		FRAME_BUFFER_WIDTH*SCREEN_HEIGHT*4



// Structs
//--------------------------------------------------
struct Vertex
{
	float u, v;
	uint32 color;
	float x, y, z;
};



struct VertexNoColor
{
	float u, v;
	float x, y, z;
};



struct VertexColor
{
	u32 color;
	float x, y, z;
};



// Globals
//--------------------------------------------------
static uint32 __attribute__((aligned(16))) list[262144];
// static uint32 __attribute__((aligned(16))) list[16384];

unsigned char* SkinTexture = NULL;
unsigned char* FontTexture = NULL;


uint32 VideoBuf;
uint32 CurBuf; 



// -------------------------------------------------
void InitGU( void )
{
	sceGuInit();

	// setup GU
	sceGuStart(GU_DIRECT,list);
	VideoBuf = 0;
	sceGuDrawBuffer(GU_PSM_8888, (void *)VideoBuf, FRAME_BUFFER_WIDTH);
	VideoBuf += FRAME_BUFFER_SIZE;
	sceGuDispBuffer(SCREEN_WIDTH, SCREEN_HEIGHT, (void *)VideoBuf, FRAME_BUFFER_WIDTH);
	VideoBuf += FRAME_BUFFER_SIZE;
	CurBuf = VideoBuf;

	sceGuOffset(2048 - (SCREEN_WIDTH/2), 2048 - (SCREEN_HEIGHT/2));
	sceGuViewport(2048, 2048, SCREEN_WIDTH, SCREEN_HEIGHT);
	sceGuScissor(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
	sceGuEnable(GU_SCISSOR_TEST);
	sceGuFrontFace(GU_CW);

	sceGuEnable(GU_TEXTURE_2D);
	sceGuShadeModel(GU_SMOOTH);
	sceGuTexWrap(GU_CLAMP, GU_CLAMP);

	// enable alpha channel
	sceGuEnable(GU_BLEND);
	sceGuBlendFunc(GU_ADD, GU_SRC_ALPHA, GU_ONE_MINUS_SRC_ALPHA, 0, 0);

	sceGuClear(GU_COLOR_BUFFER_BIT);
	sceGuFinish();
	sceGuSync(0,0);

	sceDisplayWaitVblankStart();
	sceGuDisplay(1);
}



void DeinitGU( void )
{
	sceGuTerm();
}



void BeginScene( void )
{
	sceGuStart(GU_DIRECT, list);
//	sceGuTexMode(GU_PSM_8888, 0, 0, 1);		// swizzled
	sceGuTexMode(GU_PSM_8888, 0, 0, 0);		// not swizzled
	sceGuTexFilter(GU_NEAREST, GU_NEAREST);
}



void EndScene( void )
{
	sceGuFinish();
	sceGuSync(0,0);
//	sceDisplayWaitVblankStart();
	sceGuSwapBuffers();
}



void ClearScreen( uint32 color )
{
	// set the background color
	sceGuClearColor( color );
	sceGuClear( GU_COLOR_BUFFER_BIT );
}



void DrawLine( float x0, float y0, float x1, float y1, uint32 color )
{
	struct VertexColor* vertices = (struct VertexColor*)sceGuGetMemory(2 * sizeof(struct VertexColor));

	vertices[0].color = color;
	vertices[0].x = x0; 
	vertices[0].y = y0; 
	vertices[0].z = 0.0f;

	vertices[1].color = color;
	vertices[1].x = x1; 
	vertices[1].y = y1; 
	vertices[1].z = 0.0f;

	sceGuDisable(GU_TEXTURE_2D);
	sceGuShadeModel(GU_SMOOTH);
	sceGuAmbientColor(0xffffffff);
	sceGuDrawArray(GU_LINES, GU_COLOR_8888|GU_VERTEX_32BITF|GU_TRANSFORM_2D, 2, 0, vertices);
	sceGuEnable(GU_TEXTURE_2D);

}



void DrawRect( float x_pos, float y_pos, float width, float height, uint32 color )
{
	struct VertexColor* vertices = (struct VertexColor*)sceGuGetMemory(2 * sizeof(struct VertexColor));

	vertices[0].color = color;
	vertices[0].x = x_pos; 
	vertices[0].y = y_pos; 
	vertices[0].z = 0.0f;

	vertices[1].color = color;
	vertices[1].x = x_pos + width; 
	vertices[1].y = y_pos + height; 
	vertices[1].z = 0.0f;

	sceGuDisable(GU_TEXTURE_2D);
	sceGuShadeModel(GU_SMOOTH);
	sceGuAmbientColor(0xffffffff);
	sceGuDrawArray(GU_SPRITES, GU_COLOR_8888|GU_VERTEX_32BITF|GU_TRANSFORM_2D, 2, 0, vertices);
	sceGuEnable(GU_TEXTURE_2D);
}



void DrawImage( float x_pos, float y_pos, float img_width, float img_height,	// rect pos and size on screen
				float u0, float v0, float u1, float v1,					// area of texture to render
			   int tex_width, int tex_height, int tex_buf_width, unsigned char* tex_buf, uint32 color )
{
//	printf( "Drawing image... (%f, %f), (%f, %f), (%f, %f, %f, %f, (%d, %d, %d)\n", x_pos, y_pos, img_width, img_height, u0, v0, u1, v1, tex_width, tex_height, tex_buf_width );
	sceGuTexImage(0, tex_width, tex_height, tex_buf_width, tex_buf);
	sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA); 

	float start, end;
	float cur_u = u0;
	float cur_x = x_pos;
	float x_end = x_pos + img_width;
	float slice = 32.f;
	float ustep = (u1-u0)/img_width * slice;

	// blit maximizing the use of the texture-cache
	for( start=u0, end=u1; start<end; start+=ustep )
	{
		struct Vertex* vertices = (struct Vertex*)sceGuGetMemory(2 * sizeof(struct Vertex));

		float poly_width = ((cur_x+slice) > x_end) ? (x_end-cur_x) : slice;
		float source_width = ((cur_u+ustep) > u1) ? (u1-cur_u) : ustep;

		vertices[0].u = cur_u; 
		vertices[0].v = v0;
		vertices[0].color = color;
		vertices[0].x = cur_x; 
		vertices[0].y = y_pos; 
		vertices[0].z = 0.f;

		cur_u += source_width;
		cur_x += poly_width;

		vertices[1].u = cur_u;
		vertices[1].v = v1;
		vertices[1].color = color;
		vertices[1].x = cur_x; 
		vertices[1].y = y_pos + img_height;
		vertices[1].z = 0.f;

		sceGuDrawArray(GU_SPRITES,GU_TEXTURE_32BITF|GU_COLOR_8888|GU_VERTEX_32BITF|GU_TRANSFORM_2D,2,0,vertices);
	}
}



// this is stupid.
int GetTGATextureSize( const char* file_path )
{
	int size = -1;

	SceUID fd = sceIoOpen( file_path, PSP_O_RDONLY, 0777 );

	// found it
	if( fd != -1 )
	{
		char header_data[18];
		sceIoRead( fd, header_data, 18 );	// read the header
		sceIoClose( fd );

		int width	= (short)((header_data[13]<<8) | header_data[12]);
		int height	= (short)((header_data[15]<<8) | header_data[14]);
		int bits	= header_data[16];

		// make space for our skin texture
		int bpp = bits / 8;
		size = width * height * bpp;
	}

	return size;
}

void LoadTGA( const char* file_path, unsigned char* tex_buf )
{
	// open it
	SceUID fd = sceIoOpen( file_path, PSP_O_RDONLY, 0777 );

	// found it
	if( fd != -1 )
	{
		struct TGAHeader tgaheader;
		char header_data[18];
		sceIoRead( fd, header_data, 18 );	// read the header

		tgaheader.identsize		= header_data[0];
		tgaheader.colourmaptype = header_data[1];
		tgaheader.imagetype		= header_data[2];

		tgaheader.colourmapstart	= (short)((header_data[4]<<8) | header_data[3]);
		tgaheader.colourmaplength	= (short)((header_data[6]<<8) | header_data[5]);
		tgaheader.colourmapbits		= header_data[7];

		tgaheader.xstart		= (short)((header_data[9]<<8) | header_data[8]);
		tgaheader.ystart		= (short)((header_data[11]<<8) | header_data[10]);
		tgaheader.width			= (short)((header_data[13]<<8) | header_data[12]);
		tgaheader.height		= (short)((header_data[15]<<8) | header_data[14]);
		tgaheader.bits			= header_data[16];
		tgaheader.descriptor	= header_data[17];

		// make space for our skin texture
		int bpp = (tgaheader.bits / 8);
		unsigned int file_size = tgaheader.width*tgaheader.height*bpp;

		sceIoRead( fd, tex_buf, file_size );
		sceIoClose( fd );

//		printf( "width: %d, height: %d, bpp: %d", tgaheader.width, tgaheader.height, bpp );

		// swap BGRA -> RGBA
		unsigned int i;
		for( i=0; i<file_size; i+=bpp )
		{
			tex_buf[i] ^= tex_buf[i+2] ^= tex_buf[i] ^= tex_buf[i+2];
//			printf( "(R=%3.3d, G=%3.3d, B=%3.3d, A=%3.3d)\t\t(i: %10.10d)\n", tex_buf[i], tex_buf[i+1], tex_buf[i+2], tex_buf[i+3], i );
		}
		
		// swap vertically
		int swap_idx;
		int row_size = tgaheader.width * bpp;
		for( i=0; i<(file_size/2); i+=bpp )
		{
			swap_idx = file_size - ((i/row_size+1)*row_size) + i%row_size;

			// unwound for speed.
			tex_buf[i] ^= tex_buf[swap_idx] ^= tex_buf[i] ^= tex_buf[swap_idx];
			tex_buf[i+1] ^= tex_buf[swap_idx+1] ^= tex_buf[i+1] ^= tex_buf[swap_idx+1];
			tex_buf[i+2] ^= tex_buf[swap_idx+2] ^= tex_buf[i+2] ^= tex_buf[swap_idx+2];
			tex_buf[i+3] ^= tex_buf[swap_idx+3] ^= tex_buf[i+3] ^= tex_buf[swap_idx+3];
		}
	}
}


float GetTextWidth( const char* string )
{
	const char* c = string;
	float cur_width = 0.f;
	float max_width = 0.f;

	while( *c != '\0' )
	{
		if( *c == '\n' )
		{
			cur_width = 0.f;
			c++;
			continue;
		}

		cur_width += 6.f;
		if( cur_width > max_width )
			max_width = cur_width;
		c++;
	}

	return max_width;
}

// render some text to the screen
// the font texture must start at character 32 in the ascii table (space) and
// then proceed in order to character 255. the first 32 chars are ignored because they
// have no use being rendered.
void ScreenPrint( float x, float y, const char* text, uint32 color )
{
//	sceGuTexFilter(GU_LINEAR, GU_LINEAR);

	const char* c = text;
	float offs = 0.f;

	while( *c != '\0' )
	{
		if( *c == ' ' )
		{
			offs += 6.f;
			c++;
			continue;
		}

		// move down a line 
		if( *c == '\n' )
		{
			y += 9.f;
			offs = 0.f;
			c++;
			continue;
		}

		float c_pos_x = (float)((((*c) - 32) % 32) * 6 + 1);
		float c_pos_y = (float)((((*c) - 32) / 32) * 8 + 1);

//		printf( "char [%d], c_pos_x: %f, c_pos_y: %f, offs: %f\n", *c, c_pos_x, c_pos_y, offs );

		DrawImage( x+offs, y, 6.f, 8.f, c_pos_x, c_pos_y, c_pos_x+6.f, c_pos_y+8.f, 256, 64, 256, FontTexture, color );
		offs+=6.f;
		c++;
	}

//	sceGuTexFilter(GU_NEAREST, GU_NEAREST);
}



/* Save current visible screen as PNG */
void SaveThumbnail(const char* filename, u32* vram32 )
{
//	u32* vram32;
	u16* vram16;
	int bufferwidth = 512;
	int pixelformat = PSP_DISPLAY_PIXEL_FORMAT_8888;
//	int unknown;
	int i, x, y, width=96, height=54, w_step, h_step;
	png_structp png_ptr;
	png_infop info_ptr;
	FILE* fp;
	u8* line;

	w_step = SCREEN_WIDTH / width;
	h_step = SCREEN_HEIGHT / height;

	fp = fopen(filename, "wb");
	if (!fp) return;
	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	if (!png_ptr) return;
	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr) {
		png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
		fclose(fp);
		return;
	}
	png_init_io(png_ptr, fp);
	png_set_IHDR(png_ptr, info_ptr, width, height,
		8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
		PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
	png_write_info(png_ptr, info_ptr);
	line = (u8*) malloc(width * 3);
	vram16 = (u16*) vram32;
	for (y = 0; y < SCREEN_HEIGHT; y+=h_step) {
		for (i = 0, x = 0; x < SCREEN_WIDTH; x+=w_step) {
			u32 color = 0;
			u8 r = 0, g = 0, b = 0;
			switch (pixelformat) {
				case PSP_DISPLAY_PIXEL_FORMAT_565:
					color = vram16[x + y * bufferwidth];
					r = (color & 0x1f) << 3; 
					g = ((color >> 5) & 0x3f) << 2 ;
					b = ((color >> 11) & 0x1f) << 3 ;
					break;
				case PSP_DISPLAY_PIXEL_FORMAT_5551:
					color = vram16[x + y * bufferwidth];
					r = (color & 0x1f) << 3; 
					g = ((color >> 5) & 0x1f) << 3 ;
					b = ((color >> 10) & 0x1f) << 3 ;
					break;
				case PSP_DISPLAY_PIXEL_FORMAT_4444:
					color = vram16[x + y * bufferwidth];
					r = (color & 0xf) << 4; 
					g = ((color >> 4) & 0xf) << 4 ;
					b = ((color >> 8) & 0xf) << 4 ;
					break;
				case PSP_DISPLAY_PIXEL_FORMAT_8888:
					color = vram32[x + y * bufferwidth];
					r = color & 0xff; 
					g = (color >> 8) & 0xff;
					b = (color >> 16) & 0xff;
					break;
			}
			line[i++] = r;
			line[i++] = g;
			line[i++] = b;
		}
		png_write_row(png_ptr, line);
	}
	free(line);
	png_write_end(png_ptr, info_ptr);
	png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
	fclose(fp);
}

void user_warning_fn(png_structp png_ptr, png_const_charp warning_msg)
{
	// ignore PNG warnings
}



/* Load an image into this buffer */
void LoadThumbnail(const char* filename, u32* vram32, int bufferwidth )
{
	u16* vram16;
	int pixelformat = PSP_DISPLAY_PIXEL_FORMAT_8888;
	png_structp png_ptr;
	png_infop info_ptr;
	unsigned int sig_read = 0;
	png_uint_32 width, height;
	int bit_depth, color_type, interlace_type, x, y;
	u32* line;
	FILE *fp;

	if ((fp = fopen(filename, "rb")) == NULL) return;
	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	if (png_ptr == NULL) {
		fclose(fp);
		return;
	}
	png_set_error_fn(png_ptr, (png_voidp) NULL, (png_error_ptr) NULL, user_warning_fn);
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL) {
		fclose(fp);
		png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
		return;
	}
	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, sig_read);
	png_read_info(png_ptr, info_ptr);
	png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, int_p_NULL, int_p_NULL);
	png_set_strip_16(png_ptr);
	png_set_packing(png_ptr);
	if (color_type == PNG_COLOR_TYPE_PALETTE) png_set_palette_to_rgb(png_ptr);
	if (color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8) png_set_gray_1_2_4_to_8(png_ptr);
	if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) png_set_tRNS_to_alpha(png_ptr);
	png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);
	line = (u32*) malloc(width * 4);
	if (!line) {
		fclose(fp);
		png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
		return;
	}
	vram16 = (u16*) vram32;
	for (y = 0; y < height; y++) {
		png_read_row(png_ptr, (u8*) line, png_bytep_NULL);
		for (x = 0; x < width; x++) {
			u32 color32 = line[x];
			u16 color16;
			int r = color32 & 0xff;
			int g = (color32 >> 8) & 0xff;
			int b = (color32 >> 16) & 0xff;
			switch (pixelformat) {
				case PSP_DISPLAY_PIXEL_FORMAT_565:
					color16 = (r >> 3) | ((g >> 2) << 5) | ((b >> 3) << 11);
					vram16[x + y * bufferwidth] = color16;
					break;
				case PSP_DISPLAY_PIXEL_FORMAT_5551:
					color16 = (r >> 3) | ((g >> 3) << 5) | ((b >> 3) << 10);
					vram16[x + y * bufferwidth] = color16;
					break;
				case PSP_DISPLAY_PIXEL_FORMAT_4444:
					color16 = (r >> 4) | ((g >> 4) << 4) | ((b >> 4) << 8);
					vram16[x + y * bufferwidth] = color16;
					break;
				case PSP_DISPLAY_PIXEL_FORMAT_8888:
					color32 = r | (g << 8) | (b << 16) | (255 << 24);
					vram32[x + y * bufferwidth] = color32;
					break;
			}
		}
	}
	free(line);
//	png_read_end(png_ptr, info_ptr);
	png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
	fclose(fp);
}


