//
// Copyright (C) 2006 JasonV
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#ifndef gu_funcs_h
#define gu_funcs_h



// included files
//--------------------------------------------------
#include "../types.h"



// Enumerations
//--------------------------------------------------
enum TGADescriptor
{
	TGA_DESC_H_FLIP = 1<<5,
	TGA_DESC_V_FLIP = 1<<6,
};



// Structs
//--------------------------------------------------
struct TGAHeader
{
	char  identsize;        // size of ID field that follows 18 byte header (0 usually)
	char  colourmaptype;    // type of colour map 0=none, 1=has palette
	char  imagetype;        // type of image 0=none,1=indexed,2=rgb,3=grey,+8=rle packed

	short colourmapstart;   // first colour map entry in palette
	short colourmaplength;  // number of colours in palette
	char  colourmapbits;    // number of bits per palette entry 15,16,24,32

	short xstart;           // image x origin
	short ystart;           // image y origin
	short width;			// image width in pixels
	short height;			// image height in pixels
	char  bits;				// image bits per pixel 8,16,24,32
	char  descriptor;		// image descriptor bits (vh flip bits)
};



// Functions
//--------------------------------------------------
void InitGU( void );
void DeinitGU( void );

void BeginScene( void );
void EndScene( void );


// color is in ABGR format
void ClearScreen( uint32 color );
void DrawLine( float x0, float y0, float x1, float y1, uint32 color );
void DrawRect( float x_pos, float y_pos, float width, float height, uint32 color );
void DrawImageNoBlend( float x_pos, float y_pos, float img_width, float img_height, 
			   float u0, float v0, float u1, float v1, 
			   int tex_width, int tex_height, int tex_buf_width, unsigned char* tex_buf );
void DrawImage( float x_pos, float y_pos, float img_width, float img_height, 
			   float u0, float v0, float u1, float v1, 
			   int tex_width, int tex_height, int tex_buf_width, unsigned char* tex_buf, uint32 color );



// load up some graphics
int GetTGATextureSize( const char* file_path );
void LoadTGA( const char* file_path, unsigned char* tex_buf );



// draw some text
float GetTextWidth( const char* string );
void ScreenPrint( float x, float y, const char* text, uint32 color );



// png images
void SaveThumbnail(const char* filename, uint32* vram32 );
void LoadThumbnail(const char* filename, uint32* vram32, int bufferwidth );




#endif

