/*
PMP Mod
Copyright (C) 2006 jonny

Homepage: http://jonny.leffe.dnsalias.com
E-mail:   jonny@leffe.dnsalias.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
this play the file (av output and basic functions - pause, seek ... )
*/


#include "pmp_play.h"
#include "../input.h"
#include "../skin/skin.h"
#include <pspgu.h>
#include <psprtc.h>



// Enumerations
//--------------------------------------------------
enum eHUDSelection
{
	HS_SKIP_PREV = 0,
	HS_PLAY_PAUSE,
	HS_SKIP_NEXT,
	HS_EJECT,

	HS_MAX_SELECTIONS,
};



// Globals
//--------------------------------------------------
int SelectedButton = 0;

#undef printf




void pmp_play_safe_constructor(struct pmp_play_struct *p)
	{
	p->audio_reserved = -1;

	p->semaphore_can_get   = -1;
	p->semaphore_can_put   = -1;
	p->semaphore_can_show  = -1;
	p->semaphore_show_done = -1;

	p->output_thread = -1;
	p->show_thread   = -1;
	p->input_thread  = -1;

	p->last_keyframe_pos = 0;
	p->resume_pos = 0;
	p->zoom_mode = 0;
	p->zoom_mode_prev = 0;

	p->interface_texture = 0;
	p->display_interface = 0;
	p->interface_pos = 0;

	pmp_decode_safe_constructor(&p->decoder);
	}


void pmp_play_close(struct pmp_play_struct *p)
{
	// create the icon now
	char sztemp[1024];
	strncpy( sztemp, p->resume_filename, sizeof(sztemp)-4 );
	snprintf( sztemp, sizeof(sztemp), "%s.png", p->resume_filename );
	SaveThumbnail( sztemp, p->decoder.output_frame_buffers[p->decoder.current_buffer_number].video_frame );

	// save current position to resume later (if there wasn't any error)
	SceUID	fd;
	if( (fd=sceIoOpen(p->resume_filename, PSP_O_WRONLY|PSP_O_CREAT, 0777)) )
	{
		sceIoWrite( fd, &p->last_keyframe_pos, sizeof(int) );
		sceIoWrite( fd, &p->volume_boost, sizeof(int) );
//		sceIoWrite( fd, &p->decoder.luminosity_boost, sizeof(int) );
		sceIoWrite( fd, &p->zoom_mode, sizeof(int) );
		sceIoClose( fd );

	} 
	else 
	{
		// there was an error, just delete the position file (could end up with 0 KB file)
		sceIoRemove( p->resume_filename );
	}


	if (!(p->audio_reserved < 0)) sceAudioChRelease(0);

	if (!(p->semaphore_can_get   < 0)) sceKernelDeleteSema(p->semaphore_can_get);
	if (!(p->semaphore_can_put   < 0)) sceKernelDeleteSema(p->semaphore_can_put);
	if (!(p->semaphore_can_show  < 0)) sceKernelDeleteSema(p->semaphore_can_show);
	if (!(p->semaphore_show_done < 0)) sceKernelDeleteSema(p->semaphore_show_done);

	if (!(p->output_thread < 0)) sceKernelDeleteThread(p->output_thread);
	if (!(p->show_thread   < 0)) sceKernelDeleteThread(p->show_thread);
	if (!(p->input_thread  < 0)) sceKernelDeleteThread(p->input_thread);


	pmp_decode_close(&p->decoder);


	pmp_play_safe_constructor(p);
}
	

static int pmp_wait(volatile struct pmp_play_struct *p, SceUID s, char *e)
	{
	SceUInt t = 1000000;
	

	while (1)
		{
		int result = sceKernelWaitSema(s, 1, &t);

		if (result == SCE_KERNEL_ERROR_OK)
			{
			break;
			}
		else if (result == SCE_KERNEL_ERROR_WAIT_TIMEOUT)
			{
			if (p->return_request == 1)
				{
				return(0);
				}
			}
		else
			{
			p->return_result  = e;
			p->return_request = 1;
			SelectedButton = HS_EJECT;	// let the playlist know that we want to quit viewing.
			return(0);
			}
		}


	return(1);
	}



void DrawVideoFrameScaled( volatile struct pmp_play_struct* p, int current_buffer_number )
{
	sceGuDisable( GU_BLEND );
//	sceGuTexFilter(GU_LINEAR, GU_LINEAR);

	int vid_width = p->decoder.reader.file.header.video.width;
	int vid_height = p->decoder.reader.file.header.video.height;

	float u0 = (float)((480-vid_width)>>1);
	float v0 = (float)((272-vid_height)>>1);

	// do the right zoom
	if( p->zoom_mode == ZOOM_WIDTH )
	{
		int new_height = (vid_height * 480) / vid_width;
		if( new_height > 272 )	// crop top&bottom
		{
			int	crop = (((new_height-272) * vid_width) / 480)>>1;
			DrawImage( 0.f, 0.f, 480.f, 272.f, u0, v0+crop, u0+vid_width, v0+vid_height-crop, 
				512, 512, 512, p->decoder.output_frame_buffers[current_buffer_number].video_frame, 0xFFFFFFFF );
		}
		else
		{
			DrawImage( 0.f, (272-new_height)>>1, 480.f, (272+new_height)>>1, u0, v0, u0+(float)vid_width, v0+(float)vid_height,
				512, 512, 512, p->decoder.output_frame_buffers[current_buffer_number].video_frame, 0xFFFFFFFF );
		}
	}
	else if( p->zoom_mode == ZOOM_HEIGHT )
	{
		int	new_width = (vid_width * 272) / vid_height;
		if (new_width > 480)	// crop left&right
		{
			int	crop = (((new_width-480)*vid_height)/272)>>1;
			DrawImage( 0.f, 0.f, 480.f, 272.f, u0+crop, v0, u0+(float)vid_width, v0+(float)vid_height+crop,
				512, 512, 512, p->decoder.output_frame_buffers[current_buffer_number].video_frame, 0xFFFFFFFF );
		}
		else
		{
			DrawImage( (480-new_width)>>1, 0.f, (480+new_width)>>1, 272.f, u0, v0, u0+(float)vid_width, v0+(float)vid_height,
				512, 512, 512, p->decoder.output_frame_buffers[current_buffer_number].video_frame, 0xFFFFFFFF );
		}
	}
	else if( p->zoom_mode == ZOOM_FILL )// full
	{
		DrawImage( 0.f, 0.f, 480.f, 272.f, u0, v0, u0+(float)vid_width, v0+(float)vid_height,
			512, 512, 512, p->decoder.output_frame_buffers[current_buffer_number].video_frame, 0xFFFFFFFF );
	}
	else
	{
		DrawImage( 0.f, 0.f, 480.f, 272.f, 0.f, 0.f, 480.f, 272.f,
			512, 512, 512, p->decoder.output_frame_buffers[current_buffer_number].video_frame, 0xFFFFFFFF );
	}

//	sceGuTexFilter(GU_NEAREST, GU_NEAREST);
	sceGuEnable( GU_BLEND );
}



int ShowHUD = 1; // yes... this is bad.

static int pmp_show_thread(SceSize input_length, void *input)
	{
	volatile struct pmp_play_struct *p = *((void **) input);


	unsigned int current_buffer_number = 0;



	unsigned char alpha = 0;
	ShowHUD = 1;	// show the hud on start by default

	while (p->return_request == 0)
		{
		if (pmp_wait(p, p->semaphore_can_show, "pmp_show_thread: sceKernelWaitSema failed on semaphore_can_show") == 0)
			{
			break;
			}

			if( !ShowHUD )
			{
				if( p->zoom_mode == ZOOM_NONE )
				{
					// fastest just to do a straight copy
					sceDisplayWaitVblankStart();
					sceDisplaySetFrameBuf(p->decoder.output_frame_buffers[current_buffer_number].video_frame, 512, PSP_DISPLAY_PIXEL_FORMAT_8888, PSP_DISPLAY_SETBUF_IMMEDIATE);
				}
				else
				{
					BeginScene();
					ClearScreen(SC_BLACK);
					DrawVideoFrameScaled( p, current_buffer_number );
					EndScene();
				}
			}
			else
			{
				BeginScene();
				ClearScreen(SC_BLACK);
				DrawVideoFrameScaled( p, current_buffer_number );

				int blend_start = Max(ShowHUD-60, 0 );

				alpha = 255.f-(((float)blend_start/30.f)*255.f);
				DrawHUDBG( alpha );

				DrawProgressBar( 142.f, 264.f, 198.f, 4.f, 0.f, p->decoder.reader.file.header.video.number_of_frames, p->current_video_frame, alpha );

				float fps = (float)p->decoder.reader.file.header.video.rate / (float)p->decoder.reader.file.header.video.scale;
				int	videotime = (int)(p->current_video_frame/fps);
				DrawPlayTime( videotime, alpha );
				DrawHUDButtons( SelectedButton, false, alpha );
				DrawHUDFileInfo( p->zoom_mode, p->volume_boost, alpha );
				DrawHUDBatteryInfo( alpha );

				EndScene();


				// make the HUD go away after 30 frames....
				// TODO: make this seconds instead of frames
				if( ++ShowHUD > 90 )
					ShowHUD = 0;
			}



		if (sceKernelSignalSema(p->semaphore_show_done, 1) < 0)
			{
			p->return_result  = "pmp_show_thread: sceKernelSignalSema failed on semaphore_show_done";
			p->return_request = 1;
			SelectedButton = HS_EJECT;	// let the playlist know that we want to quit viewing.
			break;
			}


		current_buffer_number = (current_buffer_number + 1) % p->decoder.number_of_frame_buffers;
		}


	return(0);
	}


static int pmp_output_thread(SceSize input_length, void *input)
	{
	volatile struct pmp_play_struct *p = *((void **) input);


    unsigned int first_video_frame     = 1;
	unsigned int current_buffer_number = 0;
	
	
	sceKernelDelayThread(1500000);


	while (p->return_request == 0)
		{
		volatile struct pmp_decode_buffer_struct *current_buffer = &p->decoder.output_frame_buffers[current_buffer_number];


		if (pmp_wait(p, p->semaphore_can_get, "pmp_output_thread: sceKernelWaitSema failed on semaphore_can_get") == 0)
			{
			break;
			}




		if (sceKernelSignalSema(p->semaphore_can_show, 1) < 0)
			{
			p->return_result  = "pmp_output_thread: sceKernelSignalSema failed on semaphore_can_show";
			p->return_request = 1;
			SelectedButton = HS_EJECT;	// let the playlist know that we want to quit viewing.
			break;
			}


		if (p->seek == 0)
			{
			sceKernelDelayThread(current_buffer->first_delay);
			sceAudioOutputBlocking(0, PSP_AUDIO_VOLUME_MAX, current_buffer->audio_frame);
			}


		if (pmp_wait(p, p->semaphore_show_done, "pmp_output_thread: sceKernelWaitSema failed on semaphore_show_done") == 0)
			{
			break;
			}




		if (first_video_frame == 1)
			{
			first_video_frame = 0;
			}
		else
			{
			if (sceKernelSignalSema(p->semaphore_can_put, 1) < 0)
				{
				p->return_result  = "pmp_output_thread: sceKernelSignalSema failed on semaphore_can_put";
				p->return_request = 1;
				SelectedButton = HS_EJECT;	// let the playlist know that we want to quit viewing.
				break;
				}
			}




		if (p->seek == 0)
			{
			int i = 1;
			for (; i < current_buffer->number_of_audio_frames; i++)
				{
				sceAudioOutputBlocking(0, PSP_AUDIO_VOLUME_MAX, current_buffer->audio_frame + p->decoder.audio_frame_size * i);
				}

			sceKernelDelayThread(current_buffer->last_delay);
			}


		current_buffer_number = (current_buffer_number + 1) % p->decoder.number_of_frame_buffers;




		while (p->return_request == 0 && p->paused == 1)
		{
			BeginScene();
			DrawVideoFrameScaled( p, current_buffer_number );

			DrawHUDBG( 255 );
			DrawProgressBar( 142.f, 264.f, 198.f, 4.f, 0.f, p->decoder.reader.file.header.video.number_of_frames, p->current_video_frame, 255 );

			float fps = (float)p->decoder.reader.file.header.video.rate / (float)p->decoder.reader.file.header.video.scale;
			int	videotime = (int)(p->current_video_frame/fps);
			DrawPlayTime( videotime, 255 );
			DrawHUDButtons( SelectedButton, true, 255 );
			DrawHUDFileInfo( p->zoom_mode, p->volume_boost, 255 );
			DrawHUDBatteryInfo( 255 );

			EndScene();

			sceKernelDelayThread(100000);
		}
		}


	return(0);
	}


static int pmp_input_thread(SceSize input_length, void *input)
{
	volatile struct pmp_play_struct *p = *((void **) input);


	InitInput();	

	int tick_resolution = sceRtcGetTickResolution();
	int TicksPerFrame = tick_resolution / 40; // 40 fps
	u64 PrevTick = 0;
	sceRtcGetCurrentTick( &PrevTick );

//	int seek_counter = 0;

	while( p->return_request == 0 )
	{
		scePowerTick(0);


		ReadInput();


		int analog = AnalogStateX();
		if( abs(analog) > 95 )
		{
			ShowHUD = 1;
			p->seek = (analog>0)?10:-10;
		}
		else if( abs(analog) > 65 )
		{
			ShowHUD = 1;
			p->seek = (analog>0)?4:-4;
		}
		else if( abs(analog) > 0 )
		{
			ShowHUD = 1;
			p->seek = (analog>0)?1:-1;
		}
		else
			p->seek = 0;


		// everything else is a button
		if( ButtonPressed(PSP_CTRL_TRIANGLE) )
		{
			p->return_request = 1;
			SelectedButton = HS_EJECT;	// let the playlist know that we want to quit viewing.
		}
		else if( ButtonPressed(PSP_CTRL_START) )
		{
			ShowHUD = 1;
			p->seek = 0;
			p->paused = !p->paused;
		}
		else if( ButtonPressed(PSP_CTRL_SELECT) )
		{
			if( ++(p->audio_stream) >= p->decoder.reader.file.header.audio.number_of_streams )
			{
				p->audio_stream = 0;
			}
		}
		else if( ButtonPressed(PSP_CTRL_CIRCLE) )
		{
			// amp audio volume
			ShowHUD = 1;
			if( ++(p->volume_boost) > 3 )
				p->volume_boost = 0;
		}
		else if( ButtonPressed(PSP_CTRL_SQUARE) )
		{
			// zoom 
			p->zoom_mode++;

			int	vid_width	= p->decoder.reader.file.header.video.width;
			int	vid_height	= p->decoder.reader.file.header.video.height;

			if( vid_width == 480 && p->zoom_mode == ZOOM_WIDTH )
				p->zoom_mode++;	// not needed -> skip !
			if( vid_height == 272 && p->zoom_mode == ZOOM_HEIGHT )
				p->zoom_mode++;	// not needed -> skip !
			if( vid_width == 480 && vid_height == 272 && p->zoom_mode == ZOOM_FILL )
				p->zoom_mode++;	// not needed -> skip !

			if (p->zoom_mode >= ZOOM_MAX)
				p->zoom_mode = 0;
		}
		else if( ButtonPressed(PSP_CTRL_LEFT) )
		{
			ShowHUD = 1;
			SelectedButton = Max( SelectedButton-1, 0 );
		}
		else if( ButtonPressed(PSP_CTRL_RIGHT) )
		{
			ShowHUD = 1;
			SelectedButton = Min( SelectedButton+1, 3 );
		}
		else if( ButtonPressed(PSP_CTRL_CROSS) && ShowHUD )
		{
			switch( SelectedButton )
			{
			case HS_SKIP_PREV:
			case HS_SKIP_NEXT:
			case HS_EJECT:
				p->return_request = 1; // just exit... will read what button was selected in the playlist loop
				break;

			case HS_PLAY_PAUSE:
				if( ShowHUD )
				{
					ShowHUD = 1; // reset to full opacity
					p->paused = !p->paused;
				}
			    break;

			default:
			    break;
			}
		}
		else if( ButtonPressed(PSP_CTRL_LTRIGGER) )
		{
			// skip to the previous file in the list
			SelectedButton = HS_SKIP_PREV;
			p->return_request = 1;
		}
		else if( ButtonPressed(PSP_CTRL_RTRIGGER) )
		{
			// skip to the next file in the list
			SelectedButton = HS_SKIP_NEXT;
			p->return_request = 1;
		}

		// throttle the input to 40 fps
		u64 current_tick;
		sceRtcGetCurrentTick( &current_tick );

		int frame_delta = current_tick - PrevTick;
		if( frame_delta < TicksPerFrame )
		{
			sceKernelDelayThread( TicksPerFrame - frame_delta );
		}
		PrevTick = current_tick;
	}

	return(0);
}


static int pmp_next_video_frame(volatile struct pmp_play_struct *p, int current_video_frame)
	{

	// save last keyframe pos
	if (p->decoder.reader.file.packet_index[current_video_frame] & 1)
		p->last_keyframe_pos = current_video_frame;

	// resume position / restart
	if (p->resume_pos != 0)
	{
		if (p->resume_pos < 0)	// restart
		{
			p->resume_pos = 0;
			return(0);
		}
		else	// resume
		{
			int	pos = p->resume_pos;
			p->resume_pos = 0;
			return(pos);
		}
	}

	if (p->seek > 0)
		{
        int number_of_skips = p->seek;

		int new_video_frame = current_video_frame + 1;
		number_of_skips--;

		while (new_video_frame < p->decoder.reader.file.header.video.number_of_frames)
			{
			if (p->decoder.reader.file.packet_index[new_video_frame] & 1)
				{
				if (number_of_skips == 0)
					{
					return(new_video_frame);
					}
				else
					{
					number_of_skips--;
					}
				}

			new_video_frame++;
			}

		return(p->decoder.reader.file.header.video.number_of_frames);
		}


	if (p->seek < 0)
		{
        int number_of_skips = -p->seek;

		int new_video_frame = current_video_frame - 1;
		number_of_skips--;

		while (new_video_frame > 0)
			{
			if (p->decoder.reader.file.packet_index[new_video_frame] & 1)
				{
				if (number_of_skips == 0)
					{
					return(new_video_frame);
					}
				else
					{
					number_of_skips--;
					}
				}

			new_video_frame--;
			}

		return(0);
		}


	return(current_video_frame + 1);
	}


char *pmp_play_start(volatile struct pmp_play_struct *p)
	{
	sceKernelStartThread(p->output_thread, 4, &p);
	sceKernelStartThread(p->show_thread,   4, &p);
	sceKernelStartThread(p->input_thread,  4, &p);




	int current_video_frame = 0;


	while (p->return_request == 0 && current_video_frame != p->decoder.reader.file.header.video.number_of_frames)
		{
		if (pmp_wait(p, p->semaphore_can_put, "pmp_play_start: sceKernelWaitSema failed on semaphore_can_put") == 0)
			{
			break;
			}


		char *result = pmp_decode_get((struct pmp_decode_struct *) &p->decoder, current_video_frame, p->audio_stream, 1, p->volume_boost);
		if (result != 0)
			{
			p->return_result  = result;
			p->return_request = 1;
			SelectedButton = HS_EJECT;	// let the playlist know that we want to quit viewing.
			break;
			}


		if (sceKernelSignalSema(p->semaphore_can_get, 1) < 0)
			{
			p->return_result  = "pmp_play_start: sceKernelSignalSema failed on semaphore_can_get";
			p->return_request = 1;
			SelectedButton = HS_EJECT;	// let the playlist know that we want to quit viewing.
			break;
			}


		current_video_frame = pmp_next_video_frame(p, current_video_frame);

		// save current video frame
		p->current_video_frame = current_video_frame;
		}

	// if the video was ended, set the resume pos to 0 so that the next time the video could restart from start
	if (current_video_frame == p->decoder.reader.file.header.video.number_of_frames)
	{
		SelectedButton = HS_SKIP_NEXT;	// move to the next file in the playlist, or quit.
		p->last_keyframe_pos = 0;
	}


	sceKernelDelayThread(1000000);
	p->return_request = 1;




	sceKernelWaitThreadEnd(p->output_thread, 0);
	sceKernelWaitThreadEnd(p->show_thread,   0);
	sceKernelWaitThreadEnd(p->input_thread,  0);


	return(p->return_result);
	}


char *pmp_play_open(struct pmp_play_struct *p, char *s, int start_from_beginning )
	{
	pmp_play_safe_constructor(p);


	char *result = pmp_decode_open(&p->decoder, s);
	if (result != 0)
		{
		pmp_play_close(p);
		return(result);
		}




	p->audio_reserved = sceAudioChReserve(0, p->decoder.reader.file.header.audio.scale, PSP_AUDIO_FORMAT_STEREO);
	if (p->audio_reserved < 0)
		{
		pmp_play_close(p);
		return("pmp_play_open: sceAudioChReserve failed");
		}


	// start the video with the pos file info, so load that shit up, yo!
	snprintf( p->resume_filename, sizeof(p->resume_filename), "ms0:/PSP/VIDEO/%s.POS", s );
	SceUID	fd;
	if ((fd = sceIoOpen(p->resume_filename, PSP_O_RDONLY, 0777)))
	{
		if( start_from_beginning )
			p->resume_pos = 0;
		else
			sceIoRead( fd, &p->resume_pos, sizeof(int) );
		sceIoRead( fd, &p->volume_boost, sizeof(int) );
//		sceIoRead( fd, &p->decoder.luminosity_boost, sizeof(int) );
		sceIoRead( fd, &p->zoom_mode, sizeof(int) );

		// make sure this thing doesn't go out of bounds
		if( p->volume_boost > 3 )
			p->volume_boost = 0;

		sceIoClose( fd );
	}

	p->semaphore_can_get = sceKernelCreateSema("can_get", 0, 0, p->decoder.number_of_frame_buffers, 0);
	if (p->semaphore_can_get < 0)
		{
		pmp_play_close(p);
		return("pmp_play_open: sceKernelCreateSema failed on semaphore_can_get");
		}


	p->semaphore_can_put = sceKernelCreateSema("can_put", 0, p->decoder.number_of_frame_buffers, p->decoder.number_of_frame_buffers, 0);
	if (p->semaphore_can_put < 0)
		{
		pmp_play_close(p);
		return("pmp_play_open: sceKernelCreateSema failed on semaphore_can_put");
		}


	p->semaphore_can_show = sceKernelCreateSema("can_show", 0, 0, 1, 0);
	if (p->semaphore_can_show < 0)
		{
		pmp_play_close(p);
		return("pmp_play_open: sceKernelCreateSema failed on semaphore_can_show");
		}


	p->semaphore_show_done = sceKernelCreateSema("show_done", 0, 0, 1, 0);
	if (p->semaphore_show_done < 0)
		{
		pmp_play_close(p);
		return("pmp_play_open: sceKernelCreateSema failed on semaphore_show_done");
		}




	p->output_thread = sceKernelCreateThread("output", pmp_output_thread, 0x8, 0x10000, 0, 0);
	if (p->output_thread < 0)
		{
		pmp_play_close(p);
		return("pmp_play_open: sceKernelCreateThread failed on output_thread");
		}


	p->show_thread = sceKernelCreateThread("show", pmp_show_thread, 0x8, 0x10000, 0, 0);
	if (p->show_thread < 0)
		{
		pmp_play_close(p);
		return("pmp_play_open: sceKernelCreateThread failed on show_thread");
		}


	p->input_thread = sceKernelCreateThread("input", pmp_input_thread, 0x4, 0x10000, 0, 0);
	if (p->input_thread < 0)
		{
		pmp_play_close(p);
		return("pmp_play_open: sceKernelCreateThread failed on input_thread");
		}




	p->return_request = 0;
	p->return_result  = 0;


	p->paused = 0;
	p->seek   = 0;


	p->audio_stream = 0;
//	p->volume_boost = 0;


	return(0);
	}
