/*
PMP Mod
Copyright (C) 2006 jonny

Homepage: http://jonny.leffe.dnsalias.com
E-mail:   jonny@leffe.dnsalias.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



// Included Files
//------------------------------------------------------------------------------
#include <pspdebug.h>
#include <psppower.h>
#include <pspusb.h>
#include <pspusbstor.h>
#include <psputility.h>
#include <psputils.h>
#include <time.h>
#include "avformat.h"
#include "me.h"
#include "me_idct.h"
#include "pmp.h"
#include "opendir.h"

#include <pspgu.h>
#include <psprtc.h>
#include "mod/gu_funcs.h"
#include "skin/skin.h"
#include "playlist.h"
#include "pmp_lib.h"
#include "input.h"
#include "types.h"



// PSP Info Setup
//------------------------------------------------------------------------------
PSP_MODULE_INFO("pmpmod", 0x1000, 1, 1);
PSP_MAIN_THREAD_ATTR(0);



// Defines
//------------------------------------------------------------------------------
//#define PROFILE__ 1



// Enumerations
//------------------------------------------------------------------------------
enum eGameState
{
	GS_INITIALIZATION = 0,
	GS_INIT_VIDEO_LIST,
	GS_READ_INPUT,
	GS_EDIT_PLAYLIST,
	GS_DRAW_PLAYLIST,
	GS_VIEW_HELP,
	GS_VIEW_CREDITS,
	GS_PLAYING,
};



// Function Declarations
//------------------------------------------------------------------------------
void print_error(char *s);



// Constants
//------------------------------------------------------------------------------
#define video_directory "ms0:/PSP/VIDEO/"
const int kFilelistMaxRows		= 10;
const int kPlaylistMaxRows		= 5;
const int kPlaylistStartingRow	= 7;	// starting point in pspDebugScreenPrintf rows



// Globals
//------------------------------------------------------------------------------
int TheGameState = GS_INITIALIZATION;
u64 PrevTick = 0;
int TicksPerFrame = 25000;	// 40 fps

int FilelistSelectedEntry	= 0;
int FilelistTopEntry		= 0;
int FilelistLastSelected	= -1;
int FilelistNumFiles		= 0;

int PlaylistSelectedEntry	= 0;
int PlaylistTopEntry		= 0;
int PlaylistLastSelected	= -1;

extern PlaylistInfo ThePlayList;
extern int FileTextScrollCounter;
extern int FileTextScrollTick;
extern int FileTextScrollDirection;

char psp_full_path[1024 + 1]; // Get the full path to EBOOT.PBP.

struct opendir_struct directory;

char**		VideoFileList = NULL;
SceIoStat** VideoStatsList = NULL;
POSInfo*	POSInfoList = NULL;

uint32 VideoWidth	= 0;	// Video Information 
uint32 VideoHeight	= 0; 
float  VideoRate	= 0; 
uint32 VideoFrames	= 0;

void*	FrameBuffer		= 0;

bool bPlaylistSelected = false;
bool bStartFromBeginning = false;
int VolumeMultiplier = 1;




// Skin Setup 
//------------------------------------------------------------------------------
void skin_load() 
{
	// make space for our skin texture
	// this is fucking retarded... when i malloc inside of LoadTGA, i lose the data by the time i go to draw
	// so i have to malloc in here. c++ is a billion times better. it's only a matter of time until i re-write this crap. again.
	extern unsigned char* SkinTexture;
	if( !SkinTexture )
	{
		const char* skin = "skin.tga";
		int data_size = GetTGATextureSize( skin );
		if( data_size > 0 )
			SkinTexture = malloc_64( data_size );
		LoadTGA( skin, SkinTexture );
	}

	extern unsigned char* FontTexture;
	if( !FontTexture )
	{
		const char* font = "raster_font_6x8.tga";
		int data_size = GetTGATextureSize( font );
		if( data_size > 0 )
			FontTexture = malloc_64( data_size );
		LoadTGA( font, FontTexture );
	}
}



void skin_unload( void )
{
	extern unsigned char* SkinTexture;
	extern unsigned char* FontTexture;
	free_64( SkinTexture );
	free_64( FontTexture );
	SkinTexture = NULL;
	FontTexture = NULL;
}



char* pmp_read_vid_header( struct pmp_lib_vid_struct *pmp_vid_header, char* s )
{
	FILE* f = fopen(s, "rb");
	if (f == 0)
	{
		printf( "pmp_lib_open: can't open file\n" );
		return("pmp_lib_open: can't open file");
	}

	if (fread(pmp_vid_header, 1, sizeof(struct pmp_lib_vid_struct), f) != sizeof(struct pmp_lib_vid_struct))
	{
		printf( "pmp_lib_open: can't read video header\n" );
		return("pmp_lib_open: can't read video header");
	}
		
	fclose(f);
	return (0);
}



// Playlist Setup
//------------------------------------------------------------------------------
void update_playlist( PlaylistInfo *play_list, int entry )
{
	if( !play_list || !play_list->play_list_entry ) 
		return;

	// just remove this entry if it's in the list, sort everything else up and return
	int i;
	for( i=0; i<play_list->number_of_entries; i++ )
	{
		if( play_list->play_list_entry[i].index == entry )
		{
			// if we have selected the last entry in the playlist, then we want to move our selection up one
			if( i >= play_list->number_of_entries-5 )
			{
				PlaylistSelectedEntry = Max( PlaylistSelectedEntry-1, 0 );
				PlaylistTopEntry = Max( PlaylistTopEntry-1, 0 );
			}

			// it's in the list so loop through everything after and move it one forward
			for( ; i<play_list->number_of_entries-1; i++ )
			{
				play_list->play_list_entry[i].index = play_list->play_list_entry[i+1].index;
			}
			play_list->play_list_entry[i].index = -1;
			play_list->number_of_entries--;

			return;
		}
	}

	// it wasn't already in the list, so just add it
	play_list->play_list_entry[play_list->number_of_entries].index = entry;
	play_list->number_of_entries++;
}



void ScrollPlaylist( int dir );
void reorder_playlist( PlaylistInfo *play_list, int entry, int dir )
{
	if( !play_list || !play_list->play_list_entry || 
		play_list->number_of_entries < 2 )
	{
		printf( "[reorder_playlist]: returned without ordering. num_entries: %d\n", play_list->number_of_entries );
		return;
	}
	
	if( entry+dir < 0 || entry+dir >= play_list->number_of_entries ) 
	{
		printf( "[reorder_playlist]: returned without ordering. num_entries: %d\n", play_list->number_of_entries );
		return;	// entry is already on top/bottom
	}

	// swap the entries
	int swap_idx = play_list->play_list_entry[entry].index;
	play_list->play_list_entry[entry].index = play_list->play_list_entry[entry+dir].index;
	play_list->play_list_entry[entry+dir].index = swap_idx;

	// move the current selection to match what we just did for rearrangement
	ScrollPlaylist( dir );
}



void UpdateSelectedVideoInfo( void )
{
	if( VideoFileList )
	{
		int selection = bPlaylistSelected ? ThePlayList.play_list_entry[PlaylistSelectedEntry].index : FilelistSelectedEntry;

		if( selection < 0 )
		{
			FilelistLastSelected = selection;
			VideoWidth = 0;
			VideoHeight = 0;
			VideoRate = 0.f;
			VideoFrames = 0;
		}
		else if( FilelistLastSelected != selection && VideoFileList[selection] ) 
		{
			FilelistLastSelected = selection;
			struct pmp_lib_vid_struct *pmp_struc = malloc_64(sizeof(struct pmp_lib_vid_struct));
			if( !pmp_struc ) 
			{
				print_error("main_loop: malloc_64 failed on pmp_struc");
				return;
			}

			if( pmp_read_vid_header(pmp_struc, VideoFileList[selection]) == 0 ) 
			{
//				printf( "%s: width: %d, height: %d, rate: %d, scale: %d, frames: %d\n", 
//					VideoFileList[FilelistSelectedEntry], pmp_struc->width, pmp_struc->height, pmp_struc->rate, pmp_struc->scale, pmp_struc->number_of_frames );

				VideoWidth = pmp_struc->width;
				VideoHeight = pmp_struc->height;
				if( pmp_struc->scale )
					VideoRate = (float)pmp_struc->rate / (float)pmp_struc->scale;
				else
					VideoRate = (float)pmp_struc->rate;
				VideoFrames = pmp_struc->number_of_frames;
			}

			free_64( pmp_struc );
		}
	}
}



void ScrollFilelist( int dir )
{
	if( !FilelistNumFiles )
		return;

	FilelistSelectedEntry = Clamp( FilelistSelectedEntry+dir, 0, FilelistNumFiles-1 );

	if( FilelistSelectedEntry == FilelistTopEntry + kFilelistMaxRows )
		FilelistTopEntry++;
	else if( FilelistSelectedEntry == FilelistTopEntry - 1 )
		FilelistTopEntry--;

	FileTextScrollCounter = 0;
	FileTextScrollTick = 0;
	FileTextScrollDirection = 1;
}



void ScrollPlaylist( int dir )
{
	if( !ThePlayList.play_list_entry || !ThePlayList.number_of_entries ) 
		return;

	PlaylistSelectedEntry = Clamp( PlaylistSelectedEntry+dir, 0, ThePlayList.number_of_entries-1 );

	if( PlaylistSelectedEntry == PlaylistTopEntry + kPlaylistMaxRows )
		PlaylistTopEntry++;
	else if( PlaylistSelectedEntry == PlaylistTopEntry - 1 )
		PlaylistTopEntry--;

	if( PlaylistSelectedEntry >= PlaylistTopEntry+5 )
		PlaylistTopEntry++;
	else if( PlaylistSelectedEntry < PlaylistTopEntry )
		PlaylistTopEntry--;

	FileTextScrollCounter = 0;
	FileTextScrollTick = 0;
	FileTextScrollDirection = 1;
}



// Utility Functions
//------------------------------------------------------------------------------
int	strcmpupr( char* s1, char* s2 )
{
	while (*s1 && *s2)
	{
		char c1 = *s1++;
		char c2 = *s2++;

		if ((c1 >= 'a') && (c1 <= 'z'))
			c1 -= 'a' - 'A';
		if ((c2 >= 'a') && (c2 <= 'z'))
			c2 -= 'a' - 'A';

		if (c1 > c2)
			return 1;
		if (c1 < c2)
			return -1;
	}

	if (*s1)
		return 1;
	if (*s2)
		return -1;

	return 0;
}



// stop the program until the user presses X or Home
void pmp_pause()
{
	while( 1 )
	{
		ReadInput();

		if( ButtonReleased(PSP_CTRL_CROSS) || 
			ButtonReleased(PSP_CTRL_HOME) )
		{
			break;
		}
	}
}



// Initialization
//------------------------------------------------------------------------------
void screen_init()
{
	pspDebugScreenSetTextColor(0xffffff);
	pspDebugScreenSetBackColor(0x893B00);
	pspDebugScreenInit();
}



void print_error(char *s)
{
	screen_init();
	pspDebugScreenSetXY(0, 0);
	pspDebugScreenPrintf("Error:\n");
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("\"%s\"\n", s);
	pspDebugScreenPrintf("\n");
	pspDebugScreenPrintf("Press X");
	pmp_pause();
}



char *static_init()
{
	scePowerSetClockFrequency(333, 333, 166);
	scePowerLock(0);

	sceCtrlSetSamplingCycle(0);
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

	av_register_all();

	if (me_struct_init() == 0)
	{
		return("static_init: me_struct_init failed");
	}

	if (me_idct_p_malloc() == 0)
	{
		return("static_init: me_idct_p_malloc failed");
	}

	return(0);
}



int init()
{
	char *result = static_init();
	if( result != 0 )
	{
        print_error(result);

		return 0;
	}

	return 1;
}



// USB
//------------------------------------------------------------------------------
void initUSBdrivers(void)
{
int exec_pu;
   
   exec_pu=sceKernelLoadModule("flash0:/kd/semawm.prx",0,NULL);
   sceKernelStartModule(exec_pu, 0, NULL, 0, NULL);

   exec_pu=sceKernelLoadModule("flash0:/kd/usbstor.prx",0,NULL);
   sceKernelStartModule(exec_pu, 0, NULL, 0, NULL);

   exec_pu=sceKernelLoadModule("flash0:/kd/usbstormgr.prx",0,NULL);
   sceKernelStartModule(exec_pu, 0, NULL, 0, NULL);

   exec_pu=sceKernelLoadModule("flash0:/kd/usbstorms.prx",0,NULL);
   sceKernelStartModule(exec_pu, 0, NULL, 0, NULL);

   exec_pu=sceKernelLoadModule("flash0:/kd/usbstorboot.prx",0,NULL);
   sceKernelStartModule(exec_pu, 0, NULL, 0, NULL);

   int ret = 0;

// FIXME: last arg should be NULL (wrong prototype in pspsdk?)
   sceUsbStart("USBBusDriver",0,0);
   ret = sceUsbStart("USBStor_Driver",0,0);
   
   if(ret)
   {
// prototype in pspsdk has only one arg, is that correct?
//      sceUsbstorBootSetCapacity(0x00800000,ret);
      sceUsbstorBootSetCapacity(0x00800000);
   }

	sceUsbActivate(0x1c8);
} 

void exitUSBdrivers(void)
{
   sceUsbDeactivate(0x1c8);

// FIXME: last arg should be NULL (wrong prototype in pspsdk?)
   sceUsbStop("USBStor_Driver",0,0);
   sceUsbStop("USBBusDriver",0,0);
} 



// File System
//------------------------------------------------------------------------------
int OpenVideoDir( void )
{
	char *result = opendir_open(&directory, video_directory);
	if( result != 0 )
	{
		pspDebugScreenSetXY(2, 4);
		pspDebugScreenPrintf("\"%s\" not found or empty", video_directory);
		pspDebugScreenPrintf("Error: %s", result);

//		exitUSBdrivers();
		opendir_close(&directory);
		pmp_pause();
		return -1;
	}

	return 0;
}



int SortVideoData( void )
{
	int	number_of_files = 0;

	int i;
	for( i=0; i<directory.number_of_directory_entries; i++ )
	{
		char*	name = directory.directory_entry[i].d_name;

		if ((strcmp( name + strlen(name) - 4, ".PMP") == 0) ||
			(strcmp( name + strlen(name) - 4, ".pmp") == 0))
			number_of_files++;
	}

	if( number_of_files )
	{
		VideoFileList = (char**)malloc_64( sizeof(char*) * number_of_files );
		VideoStatsList = (SceIoStat**)malloc_64( sizeof(SceIoStat*) * number_of_files );
		POSInfoList = (POSInfo*)malloc_64( sizeof(POSInfo) * number_of_files );

		int	n = 0;
		char sztemp[256];
		for( i=0; i<directory.number_of_directory_entries; i++ )
		{
			char	name[1024];
			strcpy( name, directory.directory_entry[i].d_name );
			int		len = strlen( name );

			if ((strcmp( name + len - 4, ".PMP") == 0) ||
				(strcmp( name + len - 4, ".pmp") == 0))
			{
				// save pointer to entry name
				VideoStatsList[n]	= &directory.directory_entry[i].d_stat;
				VideoFileList[n]	= directory.directory_entry[i].d_name;

				// try to open the POS file and set the data
				snprintf( sztemp, sizeof(sztemp), "ms0:/PSP/VIDEO/%s.POS", directory.directory_entry[i].d_name );
				SceUID fd = sceIoOpen( sztemp, PSP_O_RDONLY, 0777 );
				if( fd >= 0 )
				{
					sceIoRead( fd, &(POSInfoList[n].resume_frame), sizeof(int) );
					sceIoRead( fd, &(POSInfoList[n].volume_boost), sizeof(int) );
			//		sceIoRead( fd, &p->decoder.luminosity_boost, sizeof(int) );
					sceIoRead( fd, &(POSInfoList[n].zoom_mode), sizeof(int) );
					sceIoClose( fd );
				}
				else
				{
					POSInfoList[n].resume_frame = -1;
					POSInfoList[n].volume_boost = -1;
					POSInfoList[n].zoom_mode = -1;
//					sceIoRemove( sztemp );
				}

				// try to open the thumbnail for this file
				snprintf( sztemp, sizeof(sztemp), "ms0:/PSP/VIDEO/%s.POS.png", directory.directory_entry[i].d_name );
				fd = sceIoOpen( sztemp, PSP_O_RDONLY, 0777 );
				if( fd >= 0 )
				{
					sceIoClose( fd );
					int buf_width = 128;
					int buf_height = 64;
					POSInfoList[n].thumbnail = (unsigned int*)malloc_64( sizeof(unsigned int)*buf_width*buf_height );
					LoadThumbnail( sztemp, POSInfoList[n].thumbnail, buf_width );
				}
				else
				{
					// don't try to load it?
					POSInfoList[n].thumbnail = NULL;
				}

				n++;
			}
		}

		// bubble sort
		int	swap = 1;
		while( swap )
		{
			swap = 0;

			for( i=0; i<number_of_files-1; i++ )
			{
				if( strcmpupr(VideoFileList[i], VideoFileList[i+1]) > 0 )
				{
					swap = 1;
					
					SceIoStat *temp = VideoStatsList[i];
					VideoStatsList[i] = VideoStatsList[i+1];
					VideoStatsList[i+1] = temp;
					
					char*	name = VideoFileList[i];
					VideoFileList[i] = VideoFileList[i+1];
					VideoFileList[i+1] = name;

					POSInfo info = POSInfoList[i];
					POSInfoList[i] = POSInfoList[i+1];
					POSInfoList[i+1] = info;
				}
			}
		}
	}		

	return number_of_files;
}



void FreeVideoList( void )
{
	int i;
	for( i=0; i<FilelistNumFiles; i++ )
	{
		if( POSInfoList[i].thumbnail )
		{
			free_64(POSInfoList[i].thumbnail);
			POSInfoList[i].thumbnail = NULL;
		}
	}
	if( VideoFileList )
	{
		free_64(VideoFileList);
		VideoFileList = NULL;
	}
	if( VideoStatsList )
	{
		free_64(VideoStatsList);
		VideoStatsList = NULL;
	}
	if( ThePlayList.play_list_entry )
	{
		free_64(ThePlayList.play_list_entry);
		ThePlayList.play_list_entry = NULL;
	}
	if( POSInfoList )
	{
		free_64(POSInfoList);
		POSInfoList = NULL;
	}

	FilelistSelectedEntry	= 0;
	FilelistTopEntry		= 0;
	FilelistLastSelected	= -1;
	FilelistNumFiles		= 0;

	PlaylistSelectedEntry	= 0;
	PlaylistTopEntry		= 0;
	PlaylistLastSelected	= -1;
}



// Main Loop
//------------------------------------------------------------------------------
void main_loop()
{
	int scroll_held = 0;

	// calc frame rate throttle info
	int tick_resolution = sceRtcGetTickResolution();
	TicksPerFrame = tick_resolution / 40; // 40 fps
	sceRtcGetCurrentTick( &PrevTick );


	// run the game
	bool quit = false;
	while( 1 )
	{
		switch( TheGameState )
		{
			case GS_INITIALIZATION: 
			{
				InitInput();
				skin_load();	// load the skin
				InitGU();

				BeginScene();
				DrawSplash();
				ScreenPrint( 240.f - (GetTextWidth("Loading...")*0.5f), 160.f, "Loading...", SC_BLACK );
				EndScene();

				sceKernelDelayThread(1000000);
				TheGameState = GS_INIT_VIDEO_LIST;
				break;
			}

			case GS_INIT_VIDEO_LIST:
			{
//				initUSBdrivers();

				// see if there are any videos to play
				if( OpenVideoDir() == -1 )
				{
					printf( "could not open video list\n" );
					quit = true;
					break;
				}

				// there are videos. let's sort 'em.
				FilelistNumFiles = SortVideoData();

				// get space for the playlist
				ThePlayList.play_list_entry = NULL;
				if( FilelistNumFiles ) 
				{
					ThePlayList.play_list_entry = malloc_64(FilelistNumFiles * sizeof(PlaylistEntry));
					if (!ThePlayList.play_list_entry) 
					{
						printf("main_loop: malloc_64 failed on ThePlayList");
						return;
					}
					memset( ThePlayList.play_list_entry, -1, FilelistNumFiles * sizeof(PlaylistEntry) );
				}
				ThePlayList.list_size = FilelistNumFiles;
				ThePlayList.number_of_entries = 0;

				TheGameState = GS_DRAW_PLAYLIST;
				break;
			}

			case GS_READ_INPUT:
			{
				ReadInput();

				// quit PMP
				if( ButtonPressed(PSP_CTRL_HOME) )
				{
					quit = true;
					break;
				}

				// view credits
				if( !bPlaylistSelected && ButtonPressed(PSP_CTRL_TRIANGLE) )
				{
					TheGameState = GS_VIEW_CREDITS;
					break;
				}

				// view help
				if( ButtonPressed(PSP_CTRL_CIRCLE) )
				{
					TheGameState = GS_VIEW_HELP;
					break;
				}

				// refresh file list and clears the current playlist
				if( ButtonPressed(PSP_CTRL_SELECT) )	
				{
//					exitUSBdrivers();
					FreeVideoList();
					opendir_close(&directory);
					TheGameState = GS_INIT_VIDEO_LIST;
					break;
				}

				// clear out the pos file by removing it.
				if( !bPlaylistSelected && ButtonPressed(PSP_CTRL_LTRIGGER) )
				{
					if( FilelistNumFiles && ThePlayList.play_list_entry )
					{
						char sztemp[256];
						snprintf( sztemp, sizeof(sztemp), "ms0:/PSP/VIDEO/%s.POS", VideoFileList[FilelistSelectedEntry] );
						sceIoRemove( sztemp );

						// reset the current info for this entry
						POSInfoList[FilelistSelectedEntry].resume_frame = -1;
						POSInfoList[FilelistSelectedEntry].volume_boost = -1;
						POSInfoList[FilelistSelectedEntry].zoom_mode = -1;
						break;
					}
				}

				// toggle editing in playlist box
				if( ButtonPressed(PSP_CTRL_RTRIGGER) )
				{
					bPlaylistSelected = !bPlaylistSelected;
				}

				// play the current playlist
				if( ButtonPressed(PSP_CTRL_START) || ButtonPressed(PSP_CTRL_CROSS) )
				{
					// make sure they don't try to play an empty playlist while selecting it
					if( bPlaylistSelected && ThePlayList.number_of_entries == 0 )
						break;

					// if the user pressed start, then play the files from the beginning
					if( ButtonPressed(PSP_CTRL_START) )
					{
						bStartFromBeginning = true;
					}

					// if there aren't any files just go back to editing the file list
					if( !FilelistNumFiles || !ThePlayList.play_list_entry )
					{
						TheGameState = GS_EDIT_PLAYLIST;
						break;
					}

					// well, there are files.. so play 'em.
					TheGameState = GS_PLAYING;
					break;
				}
					
				TheGameState = GS_EDIT_PLAYLIST;
				break;
			}

			case GS_EDIT_PLAYLIST:
			{
				if( ButtonReleased(PSP_CTRL_DOWN) || ButtonReleased(PSP_CTRL_UP) )
					scroll_held = 0;

				// reorder the selected video in playlist 
				if( ThePlayList.play_list_entry ) 
				{
					// only adjust entries while holding down cross button
					if( bPlaylistSelected )
					{
						if( (ButtonPressed(PSP_CTRL_SQUARE) || ButtonHeld(PSP_CTRL_SQUARE)) )
						{
							if( ButtonPressed(PSP_CTRL_UP) )
								reorder_playlist( &ThePlayList, PlaylistSelectedEntry, -1 ); // Move entry one up
							else if( ButtonReleased(PSP_CTRL_DOWN) )
								reorder_playlist( &ThePlayList, PlaylistSelectedEntry, 1 ); // Move entry one down
						}
						else
						{
							// scroll current selection, yo!
							if( ButtonPressed(PSP_CTRL_DOWN) )
							{
								ScrollPlaylist( 1 );
							}
							else if( ButtonPressed(PSP_CTRL_UP) )
							{
								ScrollPlaylist( -1 );
							}
							// delayed continuous scrolling (delayed by 4 frames)
							else if( ButtonHeld(PSP_CTRL_DOWN) )
							{
								if( scroll_held > 3 )
									ScrollPlaylist( 1 );
								scroll_held++;
							}
							else if( ButtonHeld(PSP_CTRL_UP) )
							{
								if( scroll_held > 3 )
									ScrollPlaylist( -1 );
								scroll_held++;
							}
							else if( ButtonPressed(PSP_CTRL_TRIANGLE) )	// remove the current selection
								update_playlist( &ThePlayList, ThePlayList.play_list_entry[PlaylistSelectedEntry].index );
						}
					}
					else // File List
					{
						// scroll current selection, yo!
						if( ButtonPressed(PSP_CTRL_DOWN) )
						{
							ScrollFilelist( 1 );
						}
						else if( ButtonPressed(PSP_CTRL_UP) )
						{
							ScrollFilelist( -1 );
						}
						// delayed continuous scrolling (delayed by 4 frames)
						else if( ButtonHeld(PSP_CTRL_DOWN) )
						{
							if( scroll_held > 3 )
								ScrollFilelist( 1 );
							scroll_held++;
						}
						else if( ButtonHeld(PSP_CTRL_UP) )
						{
							if( scroll_held > 3 )
								ScrollFilelist( -1 );
							scroll_held++;
						}

						// add/remove current selection
						if( ButtonPressed(PSP_CTRL_SQUARE) )
							update_playlist( &ThePlayList, FilelistSelectedEntry );
					}
				}

				TheGameState = GS_DRAW_PLAYLIST;
				break;
			}

			case GS_DRAW_PLAYLIST:
			{
				UpdateSelectedVideoInfo();
				DrawMainMenu();
				TheGameState = GS_READ_INPUT;
				break;
			}

			case GS_VIEW_CREDITS:
			{
				static int page = 0;
				ReadInput();
				if( ButtonPressed(PSP_CTRL_CROSS) )
				{
					TheGameState = GS_DRAW_PLAYLIST;
					page = 0;
				}
				if( ButtonPressed(PSP_CTRL_LEFT) )
				{
					page = 0;
				}
				if( ButtonPressed(PSP_CTRL_RIGHT) )
				{
					page = 1;
				}

				DrawCredits( page );
				break;
			}

			case GS_VIEW_HELP:
			{
				DrawHelp();
				ReadInput();
				if( ButtonPressed(PSP_CTRL_CROSS) || ButtonPressed(PSP_CTRL_CIRCLE) )
					TheGameState = GS_DRAW_PLAYLIST;
				break;
			}

			case GS_PLAYING:
			{
				// Only leave USB connection during playback, not for just refreshing filelist
//				exitUSBdrivers();
				bool using_current_selection = false;

				// No entry was added to playlist, but start was pressed, so play the current selected file
				if( ThePlayList.number_of_entries == 0 ) 
				{
//					printf( "nothing in playlist so added current file" );
					update_playlist( &ThePlayList, FilelistSelectedEntry );
					using_current_selection = true;
				}

				// Playlist code here
				int i = 0;
				for(; i<ThePlayList.number_of_entries; ) 
				{
					// find playlist entry with index i
					int play_entry = ThePlayList.play_list_entry[i].index;
			
					BeginScene();
					DrawSplash();
					char sztext[80];
					snprintf( sztext, sizeof(sztext), "Loading: %s", VideoFileList[play_entry] );
					float text_width = GetTextWidth( sztext );
					if( text_width > 440.f ) // arbitrary width
					{
						// truncate the end and put in an elipsis
						char temp[62];
						snprintf( temp, sizeof(temp), "%s", VideoFileList[play_entry] );
						snprintf( sztext, 74, "Loading: %s...", temp ); // 74 * 6.f == 444.f (6.f is width of 1 char in font)
					}
					ScreenPrint( (240.f - (GetTextWidth(sztext)*0.5f)), 160.f, sztext, SC_BLACK );
					EndScene();

					sceKernelDelayThread(1000000);
				
#ifdef PROFILE__
					/* Clear the existing profile regs */
					pspDebugProfilerClear();
					/* Enable profiling */
					pspDebugProfilerEnable();
#endif
					// actually play the file here -----------------------------
					char* result = pmp_play( VideoFileList[play_entry], bStartFromBeginning );
					if (result != 0)
					{
						sceKernelDelayThread(1000000);
						screen_init();
						print_error(result);
						sceKernelDelayThread(1000000);
						break;
					}
#ifdef PROFILE__
					pspDebugScreenSetOffset((int)FrameBuffer);
					screen_init();
					pspDebugScreenSetXY(0, 0);
					pspDebugProfilerPrint();
					pspDebugProfilerDisable();
					pmp_pause();
#endif

					// the thumbnail may have changed, so reload it
					char sztemp[1024];
					snprintf( sztemp, sizeof(sztemp), "ms0:/PSP/VIDEO/%s.POS.png", VideoFileList[play_entry] );
					if( !POSInfoList[play_entry].thumbnail )
					{
						int buf_width = 128;
						int buf_height = 64;
						POSInfoList[play_entry].thumbnail = (unsigned int*)malloc_64( sizeof(unsigned int)*buf_width*buf_height );
					}
					LoadThumbnail( sztemp, POSInfoList[play_entry].thumbnail, 128 );

					extern int SelectedButton;
					if( SelectedButton == 0 ) // prev
					{
						i = Max(i-1, 0);
					}
					else if( SelectedButton == 3 ) // eject
					{
						break;
					}
					else
					{
						i++;
					}
				} // Playlist

				sceGuTexFlush();  // maybe we have to flush out some shit?
				bStartFromBeginning = false;

				sceRtcGetCurrentTick( &PrevTick );
				if( using_current_selection )
				{
					FreeVideoList();
					TheGameState = GS_INIT_VIDEO_LIST;
				}
				else
				{
					TheGameState = GS_DRAW_PLAYLIST;
				}
//				initUSBdrivers();
				break;
			}

			default: break;
		}

		// do we keep looping?
		if( quit )
			break;

		// throttle the game to 40 fps (note: GS_PLAYING and GS_VIEW_CREDITS states not affected).
		u64 current_tick;
		sceRtcGetCurrentTick( &current_tick );

		int frame_delta = current_tick - PrevTick;

//		printf( "done a tick. moving to the next frame in %d (tpf: %d, frame_delta: %d) ticks.\n", (TicksPerFrame - frame_delta), TicksPerFrame, frame_delta );

		if( frame_delta < TicksPerFrame )
		{
			sceKernelDelayThread( TicksPerFrame - frame_delta );
		}
		PrevTick = current_tick;
	} // while


	// we are quitting the game
	skin_load();
	BeginScene();
	DrawSplash();
	ScreenPrint( 240.f - (GetTextWidth("Goodbye!")*0.5f), 160.f, "GoodBye!", SC_BLACK );
	EndScene();
	sceKernelDelayThread(1000000);
	DeinitGU();
	skin_unload();

	FreeVideoList();
	opendir_close(&directory);
//	exitUSBdrivers();
}


int main(int argc, char *argv[])
{
	char *psp_eboot_path;
 
	strncpy(psp_full_path, argv[0], sizeof(psp_full_path) - 1);
	psp_full_path[sizeof(psp_full_path) - 1] = '\0';
 
	psp_eboot_path = strrchr(psp_full_path, '/');
	if (psp_eboot_path != NULL)
	{
		*psp_eboot_path = '\0';
	} 

	if( init() )
	{
		main_loop();
	}

	sceKernelExitGame();
	return(0);
}
