//------------------------------------------------------------------------------
// File:		skin.c
// Description: contains all the drawing functions for drawing the front end and 
//				in-game overlays.
//
// Copyright (C) 2006 JasonV
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//------------------------------------------------------------------------------



// Included Files
//--------------------------------------------------
#include "skin.h"
#include <time.h>
#include <psppower.h>
#include <psputils.h>
#include <psputility.h>
#include <psprtc.h>
#include <pspiofilemgr.h>
#include <stdio.h>
#include <string.h>

#include "../playlist.h"
#include "../mod/gu_funcs.h"
#include "../mod/pmp_play.h"



#undef printf



// Defines
//--------------------------------------------------
#define SKIN_TEX_WIDTH 256
#define SKIN_TEX_HEIGHT 64
#define SKIN_BUF_WIDTH 256

#define SKIN_FILELIST_ROWS 10
#define SKIN_PLAYLIST_ROWS 5



// Globals
//--------------------------------------------------
extern unsigned char* SkinTexture;

int g_BatteryPower = BPS_EMPTY;
int FileTextScrollCounter = 0;
int FileTextScrollTick = 0;
int FileTextScrollDirection = 1;



// Update Functions
//-------------------------------------------------------------------------------
void UpdateDateAndTime( char* out_string, int str_length )
{
	time_t tp;
	tp = sceKernelLibcTime(&tp);

	struct tm *date;
	date = gmtime(&tp);

	pspTime	clock;
	sceRtcGetCurrentClockLocalTime( &clock );
	date->tm_hour = clock.hour;
	date->tm_min = clock.minutes;
	date->tm_sec = clock.seconds;
	
	// print out the date in this fancy format... i don't care if anyone wants it any other way.
	char datefmt[64] = "\0";
	char timefmt[16] = "\0";
	int val;
	sceUtilityGetSystemParamInt(PSP_SYSTEMPARAM_ID_INT_DATE_FORMAT, &val);
	strftime( datefmt, sizeof(datefmt) , "%A - %B %e, %Y", date);

	// blink the colon between the hours and minutes every half second
	sceUtilityGetSystemParamInt(PSP_SYSTEMPARAM_ID_INT_TIME_FORMAT, &val);
	if( PSP_SYSTEMPARAM_TIME_FORMAT_24HR == val ) 
		strftime( timefmt, sizeof(timefmt), (clock.microseconds>500000?"%H %M":"%H:%M"), date);
	else
		strftime( timefmt, sizeof(timefmt), (clock.microseconds>500000?"%I %M %p":"%I:%M %p"), date);

	// print out our final string
	snprintf( out_string, str_length, "%s - %s", datefmt, timefmt );
}



void UpdateBattery( void )
{
	// cycle the battery power icons while the cable is plugged in 
	// makes the battery look like it is filling up
	if( scePowerIsPowerOnline() )
	{
		static int tick_battery_power = 0;

		pspTime	clock;
		sceRtcGetCurrentClockLocalTime( &clock );

		if( !tick_battery_power && clock.microseconds >= 500000 )
		{
			tick_battery_power = 1;
			if( ++g_BatteryPower > BPS_FULL )
				g_BatteryPower = BPS_EMPTY;
		}
		else if( tick_battery_power && clock.microseconds < 500000 )
			tick_battery_power = 0;
	}
	else 
	{
		if( scePowerIsLowBattery() || !scePowerIsBatteryExist() )
			g_BatteryPower = BPS_EMPTY;
		else
			g_BatteryPower = (scePowerGetBatteryLifePercent()*BPS_FULL)/100;
	}
}



// Drawing Functions
//------------------------------------------------------------------------------
void DrawSplash( void )
{
	// fill in the background with a color
	ClearScreen( SC_BG_MAIN );

	// draw the splash logo
	DrawImage(	198.f, 120.f, 86.f, 36.f,  0.f, 0.f, 86.f, 36.f,  SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_WHITE );
}






void DrawScrollbar( float x, float y, float window_num_items, float total_num_items, float max_bar_size, float window_top_item_index, uint32 color )
{
	// draw the scrollbar
	float bar_pos = y + 7.f;
	float bar_size = max_bar_size;
	if( total_num_items > 0.f )
	{
		bar_pos = y + 7.f + (max_bar_size / total_num_items) * window_top_item_index;

		if( max_bar_size > 0.f )
			bar_size = Clamp( (window_num_items / total_num_items * max_bar_size), 0.f, max_bar_size );
	}

	DrawImage(	x, y, 5.f, 5.f,						// top arrow
				88.f, 12.f, 93.f, 17.f, 
				SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );
	DrawImage(	x, y+max_bar_size+19.f, 5.f, 5.f,	// bottom arrow
				94.f, 12.f, 99.f, 17.f, 
				SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );
	DrawImage(	x, bar_pos, 5.f, 5.f,				// top end
				100.f, 12.f, 105.f, 17.f, 
				SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );
	DrawImage(  x, bar_pos+5.f+bar_size, 5.f, 5.f,	// bottom end
				112.f, 12.f, 117.f, 17.f, 
				SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );
	DrawImage(  x, bar_pos+5.f, 5.f, bar_size,		// filler bar
				106.f, 12.f, 111.f, 17.f, 
				SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );

}



void DrawWindow( float x, float y, float width, float height, uint32 color )
{
	DrawLine(   x+1.f,            y, x+width-1.f,            y, color );	// top 
	DrawLine(   x+1.f, y+height-1.f, x+width-1.f, y+height-1.f, color );	// bottom 
	DrawLine(           x,    y+1.f,           x, y+height-1.f, color );	// left
	DrawLine( x+width-1.f,    y+1.f, x+width-1.f, y+height-1.f, color );	// right
}



void DrawMainMenu( void )
{
	BeginScene();

	ClearScreen( SC_BG_MAIN );
	DrawHeader();
	DrawFooter();
	DrawFileList();
	DrawPlayList();
	DrawFileInfo();

	EndScene();
}



void DrawCredits( int page )
{
	BeginScene();

	DrawWindow( 0.f, 0.f, 480.f, 272.f, SC_WHITE );
	DrawRect( 1.f, 1.f, 478.f, 270.f, SC_BG_SELECTED_WINDOW );

	// draw pmp splash logo
	DrawImage(198.f, 10.f, 86.f, 36.f,  0.f, 0.f, 86.f, 36.f,  SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_WHITE );
	DrawLine( 30.f, 63.f, 450.f, 63.f, SC_WHITE );
	ScreenPrint( 159.f, 50.f, "PMPmod v2.00 (CI) by JasonV", SC_TEXT_NORMAL ); // program name and version num

	float x_pos = 20.f;
	float y_pos = 76.f;
	if( 0 == page )
	{
		// credits -------------------------------------------------------------

		// creators
		ScreenPrint( x_pos, y_pos, "PMPmod player core by jonny (http://jonny.leffe.dnsalias.com/)", SC_TEXT_NORMAL ); y_pos += 14.f;
		ScreenPrint( x_pos, y_pos, "GUI and Graphics by JasonV", SC_TEXT_NORMAL ); y_pos += 14.f;
		ScreenPrint( x_pos, y_pos, "This revision of PMPmod was built on the code-base PMPmod v1.02 (M4g3)\n\
from malloc and Raphael with graphics by argandona. This release would not\n\
have been possible without all their hard work!", SC_TEXT_NORMAL ); y_pos += 45.f;

		// codecs used
		ScreenPrint( x_pos, y_pos, "PMPmod uses libavcodec from FFMPEG (http://ffmpeg.sourceforge.net/)", SC_TEXT_NORMAL ); y_pos += 33.f;

		// special thanks
		DrawLine( 30.f, y_pos-12.f, 450.f, y_pos-12.f, SC_WHITE );
		ScreenPrint( 196.f, y_pos, "Many thanks go to:", SC_TEXT_NORMAL ); y_pos += 14.f;
		ScreenPrint( x_pos, y_pos,  "FFMPEG developers", SC_TEXT_NORMAL ); y_pos += 14.f;

		ScreenPrint( x_pos, y_pos,  "JiniCho (PMP Mod is a modified version of his original FFMPEG and\n\
FFPLAY psp port)", SC_TEXT_NORMAL ); y_pos += 23.f;

		ScreenPrint( x_pos, y_pos,  "ps2dev.org developers and users", SC_TEXT_NORMAL );

		// page 1 / 2
		ScreenPrint( 216.f, 260.f,  "Page 1/2", SC_TEXT_NORMAL );
	}
	else
	{
		// license -------------------------------------------------------------
		ScreenPrint( x_pos, y_pos, "This program is free software; you can redistribute it and/or modify it\n\
under the terms of the GNU General Public License as published by the\n\
Free Software Foundation; either version 2 of the License, or (at your\n\
option) any later version.", SC_TEXT_NORMAL ); y_pos += 48.f;

		ScreenPrint( x_pos, y_pos, "This program is distributed in the hope that it will be useful, but\n\
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY\n\
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License\n\
for more details.", SC_TEXT_NORMAL ); y_pos += 48.f;

		ScreenPrint( x_pos, y_pos, "You should have received a copy of the GNU General Public License\n\
along with this program; if not, write to the Free Software Foundation,\n\
Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n", SC_TEXT_NORMAL ); y_pos += 12.f;

		// page 2 / 2
		ScreenPrint( 216.f, 260.f,  "Page 2/2", SC_TEXT_NORMAL );
	}

	// X / O to close
	DrawLine( 1.f, 256.f, 479.f, 256.f, SC_WHITE );
	DrawImage(	397.f, 259.f, 9.f, 9.f,
		108.f, 20.f, 117.f, 29.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_WHITE );
	ScreenPrint( 412.f, 260.f, "- Close", SC_TEXT_NORMAL );

	EndScene();
}



void DrawHelp( void )
{
	BeginScene();

	// keep drawing the main menu
	ClearScreen( SC_BG_MAIN );
	DrawHeader();
	DrawFooter();
	DrawFileList();
	DrawPlayList();
	DrawFileInfo();

	// dim out the background
	DrawRect( 0.f, 0.f, 480.f, 272.f, 0xC0000000 );

	// draw the 'popup' window
	float top=29.f, left=70.f, width=340.f, height=214.f;
	DrawWindow( left, top, width, height, SC_WHITE );
	DrawRect( left+1.f, top+1.f, width-2.f, height-2.f, SC_BG_SELECTED_WINDOW );
	DrawLine( left, top+195.f, left+width, top+195.f, SC_WHITE );

	// General Usage
	float text_left = left + 6.f;
	ScreenPrint( text_left, top+  6.f, "Global:", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+ 20.f, "DPad     - Scroll Items", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+ 32.f, "Cross    - Play playlist files from saved position", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+ 44.f, "Start    - Play playlist files from beginning", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+ 56.f, "Select   - Clear current playlist", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+ 68.f, "RTrigger - Toggle editing file list / editing playlist", SC_TEXT_NORMAL );

	// file list buttons
//	DrawWindow( left+5.f, top+13.f, width-9.f, 100.f, SC_BG_SHADOW_LIGHT );
	ScreenPrint( text_left, top+ 92.f, "File List:", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+106.f, "Square   - Add / Remove file in playlist", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+118.f, "Triangle - View credits", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+130.f, "LTrigger - Clear selected file's POS file", SC_TEXT_NORMAL );

	// playlist buttons
//	DrawWindow( left+5.f, top+13.f, width-9.f, 100.f, SC_BG_SHADOW_LIGHT );
	ScreenPrint( text_left, top+154.f, "Playlist:", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+168.f, "Triangle - Remove file from playlist", SC_TEXT_NORMAL );
	ScreenPrint( text_left, top+180.f, "Square   - Hold and use DPad to re-order playlist files", SC_TEXT_NORMAL );

	// X / O to close
	DrawImage(	left+width-92.f, top+200.f, 9.f, 9.f,
				108.f, 20.f, 117.f, 29.f, 
				SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_WHITE );
	DrawImage(  left+width-63.f, top+200.f, 9.f, 9.f,
				88.f, 20.f, 97.f, 29.f, 
				SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_WHITE );
	ScreenPrint( left+width-75.f, top+201.f, "/", SC_TEXT_NORMAL );
	ScreenPrint( left+width-48.f, top+201.f, "- Close", SC_TEXT_NORMAL );

	EndScene();
}



void DrawHeader( void )
{
	// draw the bar
	DrawRect( 0.f, 0.f, 480.f, 13.f, SC_BG_TOOLBAR );
	DrawLine( 0.f, 13.f, 480.f, 13.f, SC_BG_SHADOW_LIGHT );
	DrawLine( 0.f, 14.f, 480.f, 14.f, SC_BG_SHADOW_DARK );
	DrawLine( 0.f, 15.f, 480.f, 15.f, SC_BLACK );

	ScreenPrint( 3.f, 3.f, "PMPmod v2.00", SC_TEXT_TOOLBAR );

	// draw the time
	char sztext[128];
	UpdateDateAndTime( sztext, sizeof(sztext) );
	ScreenPrint( 240.f - (GetTextWidth(sztext)*0.5f), 3.f, sztext, SC_TEXT_TOOLBAR );

	// draw the battery text info
	int battime = scePowerGetBatteryLifeTime();
	if( battime > 0 && battime < 24*60 )
		snprintf( sztext, sizeof(sztext), "%ih %02imin", battime/60, battime%60 );
	else
		snprintf( sztext, sizeof(sztext), "Charging" );
	ScreenPrint( 457.f - GetTextWidth(sztext), 3.f, sztext, SC_TEXT_TOOLBAR );

	// draw the battery icon
	UpdateBattery();
	float bat_start = 88.f + ((g_BatteryPower-1.f)*15.f);
	DrawImage(  464.f, 3.f, 14.f, 8.f,
		bat_start, 2.f, bat_start+13.f, 10.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_TEXT_TOOLBAR );
}



void DrawFooter( void )
{
	// draw footer bg
	DrawLine( 0.f, 255.f, 480.f, 255.f, SC_BLACK );
	DrawLine( 0.f, 256.f, 480.f, 256.f, SC_BG_SHADOW_LIGHT );
	DrawRect( 0.f, 257.f, 480.f, 15.f, SC_BG_TOOLBAR );

	// draw memory info
	sceIoSync("msfat0:", 1);
	uint32 buf[5];
	uint32 *pbuf = buf;
	sceIoDevctl("ms0:", 0x02425818, &pbuf, sizeof(pbuf[0]), 0, 0);

	float free_space = (float)(buf[1]*buf[3]*buf[4]) / 1024.f;	// in KB
	float size = (float)(buf[0]*buf[3]*buf[4]) / 1024.f;		// in KB

	char szunit[4];
	snprintf( szunit, sizeof(szunit), "KB" );
	if( free_space > 1024.f*1024.f ) 
	{
		free_space /= 1024.f*1024.f;
		size /= 1024.f*1024.f;
		snprintf( szunit, sizeof(szunit), "GB" );
	}
	else if( free_space > 1024.f ) 
	{
		free_space /= 1024.f;
		size /= 1024.f;
		snprintf( szunit, sizeof(szunit), "MB" );
	}
	char sztemp[256];
	snprintf( sztemp, sizeof(sztemp), "Free: %3.2f%2.2s (%d%%)", free_space, szunit, (int)(free_space*100.f/size) );
	ScreenPrint( 4.f, 261.f, sztemp, SC_TEXT_TOOLBAR );

	// get the current file's POS info
	extern POSInfo*	POSInfoList;
	extern int		FilelistSelectedEntry;
	extern int		bPlaylistSelected;
	extern int		PlaylistSelectedEntry;
	extern PlaylistInfo ThePlayList;
	int selection = bPlaylistSelected ? ThePlayList.play_list_entry[PlaylistSelectedEntry].index : FilelistSelectedEntry;

	// draw video scaling info
	DrawLine( 368.f, 256.f, 368.f, 272.f, SC_BLACK );
	DrawLine( 369.f, 256.f, 369.f, 272.f, SC_BG_SHADOW_LIGHT );
	DrawImage( 373.f, 259.f, 11.f, 11.f,					
		138.f, 14.f, 149.f, 25.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_BG_SHADOW_LIGHT );

	// draw the video scaling backing
	DrawImage( 391.f, 258.f, 5.f, 13.f,					
		150.f, 13.f, 155.f, 26.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_BG_SHADOW_LIGHT );
	DrawImage( 387.f, 262.f, 13.f, 5.f,					
		158.f, 17.f, 171.f, 22.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_BG_SHADOW_LIGHT );

	if( selection >= 0 )
	{
		// draw the video scaling values
		if( POSInfoList[selection].zoom_mode == ZOOM_WIDTH || POSInfoList[selection].zoom_mode == ZOOM_FILL )
		{
			DrawImage( 387.f, 262.f, 13.f, 5.f,					
				158.f, 17.f, 171.f, 22.f, 
				SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_TEXT_TOOLBAR );
		}
		if( POSInfoList[selection].zoom_mode == ZOOM_HEIGHT || POSInfoList[selection].zoom_mode == ZOOM_FILL )
		{
			DrawImage( 391.f, 258.f, 5.f, 13.f,					
				150.f, 13.f, 155.f, 26.f, 
				SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_TEXT_TOOLBAR );
		}
	}
	

	// draw current audio volume
	DrawLine( 403.f, 256.f, 403.f, 272.f, SC_BLACK );
	DrawLine( 404.f, 256.f, 404.f, 272.f, SC_BG_SHADOW_LIGHT );
	DrawImage( 408.f, 261.f, 9.f, 8.f, 176.f, 16.f, 185.f, 24.f,					// volume icon
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_TEXT_TOOLBAR );
//	DrawImage( 394.f, 261.f, 23.f, 8.f, 51.f, 40.f, 74.f, 48.f,					// volume bars backing
//		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_BG_SHADOW_LIGHT );

//	float vol = 0.5f; // how to get the actual volume?
//	DrawImage( 394.f, 261.f, 23.f*vol, 8.f, 51.f, 40.f, 51.f + (23.f*vol), 48.f,					// volume bars backing
//		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_TEXT_TOOLBAR );

	if( selection >= 0 )
	{
		// draw current audio multiplier
		int vol_boost = POSInfoList[selection].volume_boost;
		vol_boost = (vol_boost >= 0) ? vol_boost : 0;
		snprintf( sztemp, sizeof(sztemp), "%dx", vol_boost+1 );
		ScreenPrint( 422.f, 262.f, sztemp, SC_TEXT_TOOLBAR );
	}
	else
	{
		ScreenPrint( 422.f, 262.f, "0x", SC_BG_SHADOW_LIGHT );
	}

	// draw help button and text
	DrawLine( 436.f, 256.f, 436.f, 272.f, SC_BLACK );
	DrawLine( 437.f, 256.f, 437.f, 272.f, SC_BG_SHADOW_LIGHT );
	ScreenPrint( 453.f, 261.f, "Help", SC_TEXT_TOOLBAR );
	DrawImage( 441.f, 260.f, 9.f, 9.f, 
		88.f, 20.f, 97.f, 29.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_WHITE );
}



void DrawFileList( void )
{
	// draw file list header
	ScreenPrint( 10.f, 26.f, "Filename", SC_TEXT_NORMAL );
	ScreenPrint( 280.f, 26.f, "Size", SC_TEXT_NORMAL );
	ScreenPrint( 340.f, 26.f, "Date", SC_TEXT_NORMAL );

	uint32 box_color = SC_BG_SHADOW_LIGHT;
	extern int bPlaylistSelected;
	if( !bPlaylistSelected )
	{
		box_color = SC_WHITE;	// draw file list box outline in white cause we are selected
		
		// if selected draw the bg darker
		DrawRect( 6.f, 40.f, 469.f, 125.f, SC_BG_SELECTED_WINDOW );
	}

	DrawWindow( 4.f, 38.f, 471.f, 127.f, box_color ); // draw file list box
	DrawWindow( 461.f, 43.f, 9.f, 117.f, box_color ); // draw the scrollbar box

	// draw the scrollbar
	extern int FilelistNumFiles;
	extern int FilelistTopEntry;
	float window_num_files = 10.f;
	uint32 scrollbar_color = SC_SCROLLBAR_NOT_SELECTED;
	if( FilelistNumFiles > window_num_files && !bPlaylistSelected )
	{
		scrollbar_color = SC_SCROLLBAR_SELECTED_ENABLED;
	}
	DrawScrollbar( 463.f, 45.f, window_num_files, FilelistNumFiles, 89.f, FilelistTopEntry, scrollbar_color );

	// draw the list of files
	extern char**		VideoFileList;
	extern SceIoStat**	VideoStatsList;
	if( !VideoStatsList )
	{
		printf( "ERROR [DrawFileList()]: VideoStatsList if NULL.\n" );
		return;
	}

	extern int FilelistSelectedEntry;
	extern int FilelistTopEntry;
	int i;
	float start_y_pos = 44.f;
	uint32 text_color = SC_TEXT_NORMAL;

	int val;
	sceUtilityGetSystemParamInt(PSP_SYSTEMPARAM_ID_INT_TIME_FORMAT, &val);

	for( i=0; i<SKIN_FILELIST_ROWS && i<FilelistNumFiles; i++ )
	{
		int current_entry = FilelistTopEntry + i;
		float y_pos = start_y_pos + i*12.f;

		// set text color
		if( current_entry == FilelistSelectedEntry && !bPlaylistSelected )
		{
			text_color = SC_TEXT_HIGHLIGHT;
			DrawRect( 5.f, y_pos-1.f, 456.f, 10.f, SC_BG_HIGHLIGHT_BAR );
		}
		else if( IsInPlaylist(current_entry) )
			text_color = SC_TEXT_PLAYLIST;
		else
			text_color = SC_TEXT_NORMAL;

		// grab size of file
		char unit[4] = "KB\0";
		float size = (VideoStatsList[current_entry]->st_size / 1024.f);
		if( size > 1024.f*1024.f ) 
		{
			size /= 1024.f*1024.f;
			snprintf( unit, sizeof(unit), "GB");
		}
		else if( size > 1024.f ) 
		{
			size /= 1024.f;
			snprintf( unit, sizeof(unit), "MB");
		}

		// if the filename is too long, truncate it and draw an elipsis
		char sztext[64];
		memset( sztext, 0, sizeof(sztext) );
		if( strlen(VideoFileList[current_entry]) > 43 )
		{
			if( current_entry == FilelistSelectedEntry && !bPlaylistSelected )
			{
				strncpy( sztext, VideoFileList[current_entry]+FileTextScrollCounter, 43 );

				// scroll the selection so that you can see all of the text
				if( ++FileTextScrollTick > 5 )
				{
					int str_len = strlen( VideoFileList[current_entry] );
					if( FileTextScrollDirection )
					{
						if( ++FileTextScrollCounter > str_len-43 )
						{
							FileTextScrollCounter = str_len-43;
							FileTextScrollDirection = 0;
						}
					}
					else
					{
						if( --FileTextScrollCounter <= 0 )
						{
							FileTextScrollCounter = 0;
							FileTextScrollDirection = 1;
						}
					}

					FileTextScrollTick = 0;
				}

			}
			else
			{
				strncpy( sztext, VideoFileList[current_entry], 40 );
				snprintf( sztext, sizeof(sztext), "%s%s", sztext, "..." );
			}
		}
		else
			snprintf( sztext, sizeof(sztext), "%s", VideoFileList[current_entry] );
		ScreenPrint( 10.f, y_pos, sztext, text_color );	// filename

		snprintf( sztext, sizeof(sztext), "%6.2f%2s", size, unit );
		ScreenPrint( 280.f, y_pos, sztext, text_color );	// size

		// draw date
		int hour = VideoStatsList[current_entry]->st_mtime.hour;
		int is_pm = (hour > 11);
		if( PSP_SYSTEMPARAM_TIME_FORMAT_24HR != val ) 
			hour = (hour % 12) + 1;
		snprintf(	sztext, sizeof(sztext), "%02i/%02i/%-4i %02i:%02i %s", 
					VideoStatsList[current_entry]->st_mtime.month,
					VideoStatsList[current_entry]->st_mtime.day,
					VideoStatsList[current_entry]->st_mtime.year,
					hour,
					VideoStatsList[current_entry]->st_mtime.minute,
					(is_pm?"PM":"AM") );
		ScreenPrint( 340.f, y_pos, sztext, text_color );	// date
	} // for
}



void DrawPlayList( void )
{
	// draw title
	extern PlaylistInfo ThePlayList;

	char sztemp[64];
	snprintf( sztemp, sizeof(sztemp), (ThePlayList.number_of_entries!=1)?"Playlist - %d items":"Playlist - %d item", ThePlayList.number_of_entries );
	ScreenPrint( 180.f, 170.f, sztemp, SC_TEXT_NORMAL );

	uint32 box_color = SC_BG_SHADOW_LIGHT;
	extern int bPlaylistSelected;
	if( bPlaylistSelected )
	{
		box_color = SC_WHITE;	// draw file list box outline in white cause we are selected

		// if selected draw the bg darker
		DrawRect( 175.f, 183.f, 299.f, 66.f, SC_BG_SELECTED_WINDOW );
	}

	DrawWindow( 174.f, 182.f, 301.f, 68.f, box_color ); // draw box outline
	DrawWindow( 461.f, 187.f, 9.f, 57.f, box_color ); // draw scrollbar box

	// draw scrollbar
	extern int PlaylistTopEntry;
	float window_num_files = 5.f;
	uint32 scrollbar_color = SC_SCROLLBAR_NOT_SELECTED;
	if( ThePlayList.number_of_entries > window_num_files && bPlaylistSelected )
	{
		scrollbar_color = SC_SCROLLBAR_SELECTED_ENABLED;
	}
	DrawScrollbar( 463.f, 189.f, window_num_files, ThePlayList.number_of_entries, 29.f, PlaylistTopEntry, scrollbar_color );

	// draw playlist files
	extern char**	VideoFileList;
	extern int		PlaylistSelectedEntry;
	extern int		PlaylistTopEntry;
//	extern int		PlaylistLastSelected;
//	extern int		PlaylistNumFiles;


	if( VideoFileList && ThePlayList.play_list_entry )
	{
		float start_y = 188.f;
		int i;
		for( i=PlaylistTopEntry; i<ThePlayList.number_of_entries && i<SKIN_PLAYLIST_ROWS+PlaylistTopEntry && ThePlayList.play_list_entry[i].index>=0; i++ )
		{
			float y_pos = start_y + (12.f*(i-PlaylistTopEntry));
			uint32 text_color = SC_TEXT_NORMAL;
			if( PlaylistSelectedEntry == i && bPlaylistSelected )
			{
				text_color = SC_TEXT_HIGHLIGHT;
				DrawRect( 175.f, y_pos-1.f, 286.f, 10.f, SC_BG_HIGHLIGHT_BAR );
			}

			char sztext[64];
			memset( sztext, 0, sizeof(sztext) );
			if( strlen(VideoFileList[ThePlayList.play_list_entry[i].index]) > 46 )
			{
				if( PlaylistSelectedEntry == i && bPlaylistSelected )
				{
					strncpy( sztext, VideoFileList[ThePlayList.play_list_entry[i].index]+FileTextScrollCounter, 46 );

					// scroll the selection so that you can see all of the text
					if( ++FileTextScrollTick > 5 )
					{
						int str_len = strlen( VideoFileList[ThePlayList.play_list_entry[i].index] );
						if( FileTextScrollDirection )
						{
							if( ++FileTextScrollCounter > str_len-46 )
							{
								FileTextScrollCounter = str_len-46;
								FileTextScrollDirection = 0;
							}
						}
						else
						{
							if( --FileTextScrollCounter <= 0 )
							{
								FileTextScrollCounter = 0;
								FileTextScrollDirection = 1;
							}
						}

						FileTextScrollTick = 0;
					}
				}
				else
				{
					strncpy( sztext, VideoFileList[ThePlayList.play_list_entry[i].index], 40 );
					snprintf( sztext, sizeof(sztext), "%s%s", sztext, "..." );
				}
			}
			else
				snprintf( sztext, sizeof(sztext), "%s", VideoFileList[ThePlayList.play_list_entry[i].index] );
			ScreenPrint( 180.f, y_pos, sztext, text_color );
		}
	}
}



void DrawFileInfo( void )
{
	// draw the title
	ScreenPrint( 10.f, 170.f, "File Info", SC_TEXT_NORMAL );

	DrawWindow( 4.f, 182.f, 165.f, 68.f, SC_BG_SHADOW_LIGHT ); // draw box outline
	DrawWindow( 9.f, 188.f,  98.f, 56.f, SC_BG_SHADOW_LIGHT ); // draw the thumbnail icon box

	// draw the file info
	extern uint32 VideoWidth;
	extern uint32 VideoHeight; 
	extern float  VideoRate; 
	extern uint32 VideoFrames;

	char sztext[16];
	snprintf( sztext, sizeof(sztext), "%d x %d", VideoWidth, VideoHeight );
	ScreenPrint( 110.f, 190.f, sztext, ((VideoWidth==0) ? SC_BG_SHADOW_LIGHT : SC_TEXT_NORMAL) );
	snprintf( sztext, sizeof(sztext), "%2.2f fps", VideoRate );
	ScreenPrint( 110.f, 202.f, sztext, ((VideoRate==0) ? SC_BG_SHADOW_LIGHT : SC_TEXT_NORMAL) );

	int mins = 0;
	if( VideoRate > 0 )
		mins = (int)(VideoFrames/VideoRate)/60;
	snprintf( sztext, sizeof(sztext), ((mins==1)?"%d minute":"%d mins"), mins );
	ScreenPrint( 110.f, 214.f, sztext, ((VideoFrames==0) ? SC_BG_SHADOW_LIGHT : SC_TEXT_NORMAL) );

	// draw the divider between the bookmark and the file info
	DrawLine( 110.f, 225.f, 165.f, 225.f, SC_BG_SHADOW_LIGHT );

	// draw the bookmark time
	extern int		FilelistSelectedEntry;
	extern int		FilelistNumFiles;
	extern POSInfo*	POSInfoList;
	extern int		bPlaylistSelected;
	extern int		PlaylistSelectedEntry;
	extern PlaylistInfo ThePlayList;

	int rate = (VideoRate>0) ? VideoRate : 1;
	int selection = bPlaylistSelected ? ThePlayList.play_list_entry[PlaylistSelectedEntry].index : FilelistSelectedEntry;

	if( selection >= 0 )
	{
		// draw the bookmark icon
		DrawImage( 110.f, 230.f, 6.f, 13.f, 
			129.f, 12.f, 135.f, 25.f, 
			SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_TEXT_NORMAL );
	
		int bm_seconds	= (int)(POSInfoList[selection].resume_frame / rate) % 60;
		int bm_minutes	= (int)(POSInfoList[selection].resume_frame / rate / 60) % 60;
		int bm_hours	= (int)(POSInfoList[selection].resume_frame / rate) / 3600;
		snprintf( sztext, sizeof(sztext), "%2.2d:%2.2d:%2.2d", bm_hours, bm_minutes, bm_seconds );
		ScreenPrint( 118.f, 234.f, sztext, SC_TEXT_NORMAL );

		// draw the thumbnail icon
		if( POSInfoList[selection].thumbnail )
			DrawImage( 10.f, 189.f, 96.f, 54.f, 0.f, 0.f, 96.f, 54.f, 128, 64, 128, (unsigned char*)POSInfoList[selection].thumbnail, SC_WHITE );
	}
	else
	{
		// draw the bookmark icon
		DrawImage( 110.f, 230.f, 6.f, 13.f, 
			129.f, 12.f, 135.f, 25.f, 
			SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, SC_BG_SHADOW_LIGHT );

		ScreenPrint( 118.f, 234.f, "00:00:00", SC_BG_SHADOW_LIGHT );
	}
}



void DrawProgressBar( float x, float y, float width, float height, float start, float end, float cur_pos, unsigned char alpha )
{
	// used to draw the progress of the current video
	uint32 color = (alpha<<24) + (0x00FFFFFF & SC_BG_SHADOW_LIGHT);

	DrawImage(		 x-6.f, y, 5.f, 4.f, 129.f, 27.f, 134.f, 31.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );	// left end piece
	DrawImage( x+width+1.f, y, 5.f, 4.f, 135.f, 27.f, 140.f, 31.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );	// right end piece
	DrawRect( x, y, width, height, color );	// bar backing


	float range = ((end - start) > 0.f) ? (end - start) : 1.f;
	float prog_width = width * cur_pos / range;
	color = (alpha<<24) + (0x00FFFFFF & SC_RED);
	DrawImage( x+prog_width-1.f, y-2.f, 3.f, 2.f, 141.f, 27.f, 144.f, 29.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );	// notch
	color = (alpha<<24) + (0x00FFFFFF & SC_BG_SHADOW_DARK);
	DrawRect( x+1.f, y+1.f, prog_width, height/2.f, color );
}



void DrawHUDBG( unsigned char alpha )
{
	uint32 color = (alpha<<24) + (0x00FFFFFF & SC_WHITE);
	DrawImage(   0.f, 248.f,  16.f, 24.f,  0.f, 39.f,  16.f, 63.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color ); // left corner
	DrawImage(  16.f, 248.f,  96.f, 24.f, 17.f, 39.f,  33.f, 63.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color ); // left middle
	DrawImage( 112.f, 248.f,  32.f, 24.f, 34.f, 39.f,  66.f, 63.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color ); // left angle down
	DrawImage( 144.f, 248.f, 192.f, 24.f, 67.f, 39.f,  83.f, 63.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color ); // middle middle
	DrawImage( 336.f, 248.f, 144.f, 24.f, 84.f, 39.f, 228.f, 63.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color ); // right side
}



void DrawPlayTime( int time, unsigned char alpha )
{
	char sztemp[16];
	snprintf( sztemp, sizeof(sztemp), "%2.2d:%2.2d:%2.2d", (time/3600), ((time/60)%60), (time%60) );
	uint32 color = (alpha<<24) + (0x00FFFFFF & SC_TEXT_TOOLBAR);
	ScreenPrint( 78.f, 261.f, sztemp, color );
}



void DrawHUDButtons( int selected, bool paused, unsigned char alpha )
{
	uint32 color_high = (alpha<<24) + (0x00FFFFFF & SC_BUTTON_HIGHLIGHT);
	uint32 color_norm = (alpha<<24) + (0x00FFFFFF & SC_BUTTON_NORMAL);
	DrawImage( 2.f, 260.f, 18.f, 10.f, 148.f, 2.f, 166.f, 12.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, (selected==0)?color_high:color_norm ); // skip back

	if( paused )
		DrawImage( 19.f, 260.f, 18.f, 10.f, 165.f, 2.f, 183.f, 12.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, (selected==1)?color_high:color_norm ); // play
	else
		DrawImage( 19.f, 260.f, 18.f, 10.f, 182.f, 2.f, 200.f, 12.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, (selected==1)?color_high:color_norm ); // pause

	DrawImage( 36.f, 260.f, 18.f, 10.f, 216.f, 2.f, 234.f, 12.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, (selected==2)?color_high:color_norm ); // skip ahead
	DrawImage( 53.f, 260.f, 18.f, 10.f, 233.f, 2.f, 251.f, 12.f, SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, (selected==3)?color_high:color_norm ); // eject (exit playback)
}



void DrawHUDFileInfo( int stretch, int audio_amp, unsigned char alpha )
{
	// draw video scaling info
	uint32 color = (alpha<<24) + (0x00FFFFFF & SC_TEXT_TOOLBAR);
	uint32 color_bg = (alpha<<24) + (0x00FFFFFF & SC_BG_SHADOW_LIGHT);
	DrawImage( 353.f, 259.f, 11.f, 11.f, 138.f, 14.f, 149.f, 25.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );

	// draw the video scaling backing
	DrawImage( 369.f, 258.f, 5.f, 13.f, 150.f, 13.f, 155.f, 26.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color_bg );
	DrawImage( 365.f, 262.f, 13.f, 5.f, 158.f, 17.f, 171.f, 22.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color_bg );

	// draw the video scaling values
	if( stretch == ZOOM_WIDTH || stretch == ZOOM_FILL )
	{
		DrawImage( 365.f, 262.f, 13.f, 5.f, 158.f, 17.f, 171.f, 22.f, 
			SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );
	}
	if( stretch == ZOOM_HEIGHT || stretch == ZOOM_FILL )
	{
		DrawImage( 369.f, 258.f, 5.f, 13.f, 150.f, 13.f, 155.f, 26.f, 
			SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );
	}

	// volume icon
	DrawImage( 386.f, 260.f, 9.f, 8.f, 176.f, 16.f, 185.f, 24.f,
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );

	// draw current audio multiplier
	char sztemp[4];
	audio_amp = (audio_amp >= 0) ? audio_amp : 0;
	snprintf( sztemp, sizeof(sztemp), "%dx", audio_amp+1 );
	ScreenPrint( 398.f, 261.f, sztemp, color );
}



void DrawHUDBatteryInfo( unsigned char alpha )
{
	char sztext[16];
	int battime = scePowerGetBatteryLifeTime();
	if( battime > 0 && battime < 24*60 )
		snprintf( sztext, sizeof(sztext), "%ih %02imin", battime/60, battime%60 );
	else
		snprintf( sztext, sizeof(sztext), "Charging" );
	uint32 color = (alpha<<24) + (0x00FFFFFF & SC_TEXT_TOOLBAR);
	ScreenPrint( 432.f, 261.f, sztext, color );

	// draw the battery icon
	UpdateBattery();
	float bat_start = 88.f + ((g_BatteryPower-1.f)*15.f);
	DrawImage(  416.f, 261.f, 14.f, 8.f,
		bat_start, 2.f, bat_start+14.f, 10.f, 
		SKIN_TEX_WIDTH, SKIN_TEX_HEIGHT, SKIN_BUF_WIDTH, SkinTexture, color );
}



