//------------------------------------------------------------------------------
// File:		skin.h
// Description: contains all the drawing functions for drawing the front end and 
//				in-game overlays.
//
// Copyright (C) 2006 JasonV
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//------------------------------------------------------------------------------

#ifndef __SKIN_H__
#define __SKIN_H__



// Included Files
//--------------------------------------------------
#include "../mod/gu_funcs.h"
#include "../types.h"



// Enumerations
//--------------------------------------------------
enum eBatteryPowerStates
{
	BPS_EMPTY = 1,
	BPS_1_BAR,
	BPS_2_BAR,
	BPS_FULL,
};



//							    A B G R
enum eSkinColors
{
	SC_WHITE				= 0xFFFFFFFF,
	SC_BLACK				= 0xFF000000,

	SC_RED					= 0xFF0000FF,

	// bg colors
	SC_BG_TOOLBAR			= 0xFFE8E8E8,
	SC_BG_SHADOW_LIGHT		= 0xFFA8A8A8,
	SC_BG_SHADOW_DARK		= 0xFF585858,
	SC_BG_MAIN				= 0xFF606060,
	SC_BG_SELECTED_WINDOW	= 0xFF404040,
	SC_BG_HIGHLIGHT_BAR		= SC_BG_SHADOW_LIGHT,

	// highlight colors
	SC_SCROLLBAR_SELECTED_ENABLED	= 0xFF00FF00,
	SC_SCROLLBAR_SELECTED_DISABLED	= 0xFF407F40,
	SC_SCROLLBAR_NOT_SELECTED		= SC_SCROLLBAR_SELECTED_DISABLED,

	// text colors
	SC_TEXT_TOOLBAR			= SC_BLACK,
	SC_TEXT_NORMAL			= 0xFF00FF00,
	SC_TEXT_HIGHLIGHT		= SC_BLACK,
	SC_TEXT_PLAYLIST		= 0xFF00FFFF,

	// hud buttons
	SC_BUTTON_NORMAL		= 0xFFFFFFFF,
	SC_BUTTON_HIGHLIGHT		= 0xFFFFB951,
};



// Update Functions
//--------------------------------------------------
void UpdateDateAndTime( char* out_string, int str_length );
void UpdateBattery( void );




// Drawing Functions
//--------------------------------------------------
void DrawSplash( void );

void DrawScrollbar( float x, float y, float window_num_items, float total_num_items, float max_bar_size, float window_top_item_index, uint32 color );
void DrawWindow( float x, float y, float width, float height, uint32 color );

void DrawMainMenu( void );
void DrawCredits( int page );
void DrawHelp( void );

void DrawHeader( void );
void DrawFooter( void );
void DrawFileList( void );
void DrawPlayList( void );
void DrawFileInfo( void );


// hud drawing -------------------------------------
void DrawProgressBar( float x, float y, float width, float height, float start, float end, float cur_pos, unsigned char alpha );
void DrawHUDBG( unsigned char alpha );
void DrawPlayTime( int time, unsigned char alpha );
void DrawHUDButtons( int selected, bool paused, unsigned char alpha );
void DrawHUDFileInfo( int stretch, int audio_amp, unsigned char alpha );
void DrawHUDBatteryInfo( unsigned char alpha );



#endif // __SKIN_H__
